'''
Function:
    Implementation of OpenAIChatGPTEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from .base import BaseEndpoint
from ..utils import ChatRequest, ChatResponse, ModelIOType, Modality


'''SETTINGS'''
CHATGPT_FREE_API_KEYS = {
    'gpt-4o': [],
    'gpt-4o-mini': [],
}
CHATGPT_ANYWHERE_FREE_SHARED_KEYS = [
    'jw30KfjhzWiCv_ebwm3R5hRLVOX8IY9fvWUgHsL25Uj4qmv6XXFuuUOQWMOrJS5NDy5uLcwenJZ59Z3I4H8UkT0CnOU_UKE41K41qSkOm9qu5jg_G4sRWreeguM8MSTb', 
    '6sNVVAwcrhBAP0IJ5E06lrgevg4o2QSn_ibLLXdsXZjUkWdYwKxTOOkCn7dhS3luhVyd-UACwzjojOnOGrtaSnbodksrUU8fHUe91UiWzVJmi50GmOUDXV7s0cpF4hFE', 
    'BKTdCN1PTE3kOH9M28wnfAHO1au9WtjssL6zyH60mocGS-Hf8loZqH0Q2Bsp0DKr0-MGR6I7ms_yofpTQfR_wPkhTi5_hDfulXEm5lzH7CmNOBXKhL2xjWyxtQ3mbyuC', 
    'KLfM9uA3eVACCpXHBDGH1t1LARLRiOjHh8vpq0d3Vx8wV-jYQKIFggtw3XgdJA68VbJJzqflv6CnXG98a-GWQvxq_4Lf0kUr5dFVfraxdLV-1BnnAGJqR4wQgWoDQzWb', 
    'gJloHBPdzYBLaAWkRsdS9YP2HjyE2XLTmYZL6JQl2PfyeeCuSHmt8Q70015oWCHWFTP4dqc4eQwSmNEAoKmgVrI3JuhLbMtvz8VQIDffbsbig7UxfJgu4znZKEJh-R6i', 
    'plMXcvnACsKl6kxu1LDOdd6rVaqNBxhsper59meUf-Xf9RvOJ6B4ZqM8poKw5Vl1EsYPE9_UO5p9IF1Il7PMNzaNaX8irzyQoULZNqQbVMc-aP37iZmyWqe2y_UlTk7T', 
    'O23Drzh2YoDuAF1Yv8x2RDVLw2-wTNNYzB5jkaXf3Z4KBVQE6HLa-PxJRWvIb2k9CkXVszCbgufwNkUYqC-nvs9DqjmuOjDjui_cYSZ74yzXBcfOW3cL9GiaT_09LciX', 
    'TvSrB-wacUcTkYBUucmRlJT2afSf0ykv01wsog9xXSZOhfAiPG21w4SOtL7U_0p0_gLWXgN_GfLuuTaYnF7xssxr1sC13YsqNt0758DhaRx2d_CpxgQ5RDqx4W94HHsJ',
]
CHATGPT_ANYWHERE_FREE_API_V1_KEYS = {
    'gpt-5.2': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-5.1': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-5': CHATGPT_ANYWHERE_FREE_SHARED_KEYS,
    'gpt-4o': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-4.1': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-4o-mini': CHATGPT_ANYWHERE_FREE_SHARED_KEYS,
    'gpt-3.5-turbo': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-4.1-mini': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-4.1-nano': CHATGPT_ANYWHERE_FREE_SHARED_KEYS,
    'gpt-5-mini': CHATGPT_ANYWHERE_FREE_SHARED_KEYS, 'gpt-5-nano': CHATGPT_ANYWHERE_FREE_SHARED_KEYS,
}
CHATGPT_BIANXIE_FREE_API_V1_KEYS = {
    'gpt-4o': ['rXftowCdzp52RYbrejAJ17TkqTHqibdMBbaOsEbYLppnPONitvK7YqQQBcf9hWaiiv85TAgmWotshy4LOCqaBDn6ib10rOX9G4-jlq_qGQq3EZoRsMwsju0KEipRWRgK']
}
CHATGPT_GPTBESTVIP_FREE_API_V1_KEYS = {
    'gpt-4o-mini': ['S06SosGNrr-UaXY46KfjK-2sltumTl21BhuivUpdXp36ZDi7xnoCpebdmGHhTzpu_RZB6ZgBq0WfwPBJ45o9qH7GlrwZh43LD1eiPSgk0y2E1JreXxT20O3F2zx_0lqD'],
    'gpt-4o': ['S06SosGNrr-UaXY46KfjK-2sltumTl21BhuivUpdXp36ZDi7xnoCpebdmGHhTzpu_RZB6ZgBq0WfwPBJ45o9qH7GlrwZh43LD1eiPSgk0y2E1JreXxT20O3F2zx_0lqD'],
    'gpt-5.1': ['S06SosGNrr-UaXY46KfjK-2sltumTl21BhuivUpdXp36ZDi7xnoCpebdmGHhTzpu_RZB6ZgBq0WfwPBJ45o9qH7GlrwZh43LD1eiPSgk0y2E1JreXxT20O3F2zx_0lqD'],
    'gpt-5.2': ['S06SosGNrr-UaXY46KfjK-2sltumTl21BhuivUpdXp36ZDi7xnoCpebdmGHhTzpu_RZB6ZgBq0WfwPBJ45o9qH7GlrwZh43LD1eiPSgk0y2E1JreXxT20O3F2zx_0lqD'],
}


'''OpenAIChatGPTEndpoints'''
class OpenAIChatGPTEndpoints(BaseEndpoint):
    provider, model = "openai", "chatgpt"
    def __init__(self, **kwargs):
        super(OpenAIChatGPTEndpoints, self).__init__(**kwargs)
        T2T = ModelIOType.fromtag("T2T")
        TI2T = ModelIOType.create(inputs=[Modality.TEXT, Modality.IMAGE], outputs=[Modality.TEXT])
        for ver in list(CHATGPT_ANYWHERE_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[T2T, TI2T] if ver not in ['gpt-3.5-turbo'] else [T2T])
            self.registerapi(version=ver, name="chatanywhereapiv1", io=T2T, handler="chatanywhereapiv1", priority=10, note='thirdpart api: https://api.chatanywhere.tech/v1')
        for ver in list(CHATGPT_BIANXIE_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[T2T, TI2T])
            self.registerapi(version=ver, name="bianxieapiv1", io=T2T, handler="bianxieapiv1", priority=11, note='thirdpart api: https://api.bianxie.ai/v1')
            self.registerapi(version=ver, name="visionbianxieapiv1", io=TI2T, handler="visionbianxieapiv1", priority=11, note='thirdpart api: https://api.bianxie.ai/v1')
        for ver in list(CHATGPT_GPTBESTVIP_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[T2T, TI2T])
            self.registerapi(version=ver, name="gptbestvipapiv1", io=T2T, handler="gptbestvipapiv1", priority=11, note='thirdpart api: https://hk-api.gptbest.vip/v1')
            self.registerapi(version=ver, name="visiongptbestvipapiv1", io=TI2T, handler="visiongptbestvipapiv1", priority=11, note='thirdpart api: https://hk-api.gptbest.vip/v1')
    '''chatanywhereapiv1'''
    def chatanywhereapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://api.chatanywhere.tech/v1', candidate_api_keys=CHATGPT_ANYWHERE_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''bianxieapiv1'''
    def bianxieapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://api.bianxie.ai/v1', candidate_api_keys=CHATGPT_BIANXIE_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''visionbianxieapiv1'''
    def visionbianxieapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.visionopenaisdk(
            base_url='https://api.bianxie.ai/v1', candidate_api_keys=CHATGPT_BIANXIE_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''gptbestvipapiv1'''
    def gptbestvipapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://hk-api.gptbest.vip/v1', candidate_api_keys=CHATGPT_GPTBESTVIP_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''visiongptbestvipapiv1'''
    def visiongptbestvipapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.visionopenaisdk(
            base_url='https://hk-api.gptbest.vip/v1', candidate_api_keys=CHATGPT_GPTBESTVIP_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )


'''
Function:
    Implementation of MiniMaxEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import secrets
from .base import BaseEndpoint
from typing import Union, Dict, Any, Optional
from ..utils import ChatRequest, ChatResponse, ModelIOType, StreamSanitizer, Modality


'''SETTINGS'''
MINIMAX_FREE_SHARED_KEYS = [
    'RxSEewtC765SQ2a31yZz-jfVWc2n_N53k0MhVxtmHEJtpipRsdLE_7qWBROHslXTbUHXo2-7MowCk-eCZC4uFnn0f6l_1eZodKcLL2-fwFjpwKO8ZzXh7qQYUCciAMM845IdQ_nqGa-etULLGkcjqj1RBlVraohWHwgE_qqAERLCnjj4IWclKXWCBo4O1krmUjj-9nE_CDWwnERCiPBDy1lnx7c3wH2dL-H7yg5rJ55CqMhhwkpy0UHGMedjAF3MGU119ptN-2Ravu0pFuUNJeSMzyWl7bbnqoNjb-leGAAbl8exsrDWBPVvo1q59P5Yo3wu7QI59liUY6l9xmNoyV_25UxO5xI696pyQKdiql9xtgkkalVomAXZ2aHJkZ97V4ZAdH9gpe0hGHG3Q2BAS3XQAnhVAxxWxDL1VoyS9tG--ZDjc0Z0XuGWLccT1CZFxc0o6NQOx2SJ9fqiZgHi9vRqjbCjAroSTfdc4zhbu3iemfRFupOqHNP-uBLUazrQvtkCiH5m_fydC0HASXgnrv8mzcxskqcT50bgaNcGCNNWNGl_PgMJ1K_RCoZdw16gFY_fvG4kBkWOKZZyfBdFCMwiCFEK09q4kKs6aOt-LhHyplA6FxcWFLEBDw_zM1-BEYXR7nUFFQwsOfZWFox5G8AACkxiIl_rLFcTSIgMV2EPEHpin91wqOBBfjv5MuIOeXyoit35L7CLekm45k2hIPgoh72lbbplv_NLQK93wseSuVAz9Q6cUis5tu82HVCC0fOr7KjI0HvT7ECi_b9iuTV4i5wxF2wa23uC9cfRWPLn84ROLw3rMjTqinKh_q1E6z8k66Hc75J0HCA574s_sHL7HUL33GQbZyorhP1kTQhe6__mQTvPj6_iwvhPHIGgfnS2YgA5x8nHFU6GlxzdVGw-HtsC82h82NE1mbcpjc349yDIxlBTUxvp_iAMyVyzBSJBZQLvIeRBArC6CeKEtOxkCLNHdPgg04MRYYtmS94goA5UP80xqVyQTKdnbCunnT0DCx4JeBOrD7fO-PQ8bZ7RmYfPRT4qCsvGFkDQFoYanNhCqNIqlOd4Cim1JgElHdJn9UlKXgyNYM_UMaDcpgEzhvuS2cxexoOEFaVrzRQMhJPK82JQWSsSUyn0Gx8y4pgu4nzSNDb0A23qzUii2oZT06gfwKSHlrAQeWL3prtSGGnKMcMc77n6MHU3NixBvVJN3waItMYlVINfaCMaA_fpcijMWlnTmepCToYVo9KBP88dE4NOTtd0fznTgkG8',
    'MTNRE4D0LSf6eyMsNEypUMfr05Zg2vj7j2RrlhGtNvLrbQbeSD5m_WyuDdZrywjAY1UPjrfow3RMW3Q3oLf-9pKa5RBL0kRNg3xA9M8No8CJ595m0E2FtDYx6gQ4HTfutUoMwfSvtDM6QHuiJuiqgvpsGnBIdCkSP2hHsT5rkQWRBm_CRKL2EgKibZZ3DjHhm4eViriqaPGW-NUvjjuaU-wrbJjuWRHZ4knC2k7juDQjWmJTsvT2JabHdEcO7DVEz7xKJVLEDFi2hcsVN1cdohnxxBg77FVvxqCh5fSOAv0_TIoEofFWv04lT3ENsZeI-pmvIUnc3xIepqh8-jl_TMSW1zDNJY92YSSSiDTgsCPXELIVDWTi3NZ6KIowKJVRjZbQzAVJmFC-BTsxAg3XRNqrjmu8GmbTQghgi64h82Ztx853-cgKXnQoOMfNNreLhCGKWGFsSm79GLYGCltkpmSO8afPkLzyvZ-pPK2DN3GGnidwkAue9vcw_vrPIwuSika3-TcClcZDoDMtJ3MAGcLttdW7_I80dtH7D6r1nkHzEXgMl5Dm6OwxdR45D8bENivc0gLmKvDsaCL9of4EIYE6NO3V_-OTT7pxvJ8m90HF-ou6AWUh9q056Eg4NUkUgIhEWvTtHss97dmfj93XsyVb4h4TCrvyp8ZyQIpFqrm_7eqQevn_J1PYHB47VFnNJDF2p1UBZgbt7C_5VUpe7aUROydnE6nfT-mRAsRifYdYKVGWjpGz-agWF08TBI7K-cmv2NMdPFMPPbCx48uEX5lDNaJOxpbf1zTvyPY4lY0R3f7KSzTtZiDweyldYSsin4sZuwg7XvOZtPkgO93xdYvsfIZyjDVw94eCeILad9fazu-EQi4rZK8ZLbtHeydKhhswXrHHC9okZ6L6yOFmbzxPZ91hchCY7ifuk4Ho5XB587rEJoKAD0_W4nFnv3qEQm3uJp4nxncUatzXOF4XmOpBH2nPz7chOt3dkkW_ieQQlSK65iJEiKTuZ-FXMiLpB2pU9vgOsMZwKjxfu-0HdPCjXMHsiKPsZQnlAtqn6jyJhzqgoYAmAzAMjvGt1KxcURJHhc1Q_CWquop3CGrzWY_ju1tLWTIAr7v4UfgBdY8jFU1FiwHp8hPu5iRbXuQOnacKb8kPGKCi_---eIM-1hrvjpD9M7vCiUG-o91ZAbki8Ibu9sP3bahfq-ZDvUMJS773Zfkl0orAzW4rhvBRpGLj5ruQWNrWH2jwmHOGEsOVSeMOn1Wisp4KyhVGNFyd',
    'j8CCnjWwc5UOxXdW_lfDFFvxnRIiGpPjIiWA0OKHPrCk6ba_GDGR051QsCm6C2UHPwe2PrCNA5BEfRST4CorwJbmuJUyBL2DqWikJ48KFqEdeQVU_KfKTfiGto47lFu4T2Z1OzOmAyxltdoNH3UCm9W9ehmFCvorvjIlyVgv3wKpc9ewkP3zhswLiaeW7_h0_SWMuY-qALzVliz693-uwteJ06Vj6Z8DzvyLeljb9YFkcTcb2dqleHVO55xoGj5uZTHaX38qg7vyjWLs9AP7FSyM9zavyTu-hm7G5tHMpMGbC99Z2iKX0Sn2m2cA4O_4CnG1JB0CEWn4D-fFumvnJLscWSw19cfoON2xX6eg6LUE9-98U1Z4UsChFHqpjgy3Uf_WQ7sTj9RZZrY0iH9T7rLN3Xdg0CP4Jer2C8J6Xl5DiYTFSVKyU_yIAzDSca3Y0nfZvlBWvZeoao_wAa1SpKqKJe0UR8wxLyai0_Efx_Bu5MSInSNwUgosphq3NyJUiOenPasMOQnKJXmS36FsQTQdFVpyL5fxqW5FgxkQ9oh43RfkDIz6iMNaZOq1r3bey2H5IDKKUQLQPFxx3g4U5ljFuywpBpP0icnnJrFuqGQ2cdFzg7nGO6ZIQPX3vLEU4dxOVkzmz_FnqT7lxeTA9F_gp1mUjdFamK4MDaS09uYo8HP7qWm6J76xbMrDtfa71A0mRujWGroRsCj2HN6OjMP13-P7b387tfDsPw79wsqIpwp6AuqgYscX4vJAetjJVMc-b0BkwWnOb2X_lx-1Qbjcxb3RlJA08Ir6Bqo32iJy-H8h6Q7fmzCcJlrcB2IZE5tDzoaaNMwPExj7kGLlS2z8Q4rytXHIqMWXqX6oc0WmWDOZzgtuzThPcbIvxLSuh8UzgCzTv0jSa_JuTTucjEM31_DcOmzJZ_Tl927Y2MVabCfPZJAzgmluJ9c8sOZOf1FF0Q_SIjIv6kxOTC-bB1r6ovG32b3U4DBFIWxjqEHc66MLDClUHuyIBcRtXQmVtOxrG08dTpFsleXed1XPhZkbIOvB4Ll2-HXbT_IwHjQO3-e808tBLfuqikr-dz2XL7VlMfZvF_V_LDxkezBfn6V2vuCYJ5c_1bomQTA2T6lY104QF5wQk2MPdQDgVRwpVtZzH-m4mfWy7j_JdroIyQ7uPbj3_r7WhnVFI_k-gJCjyxj38zlB6uwKSYo24-UrLTTw04o6QkJqJY5eKG31d9lA5_nzWnAMMIw6FQ95uQ2jujEQtelIfsFaj5rI3ykaEXs-4XBiayj4Qrxo',
]
MINIMAX_FREE_API_V1_KEYS = {
    'MiniMax-M2': MINIMAX_FREE_SHARED_KEYS, 'MiniMax-M2-Stable': MINIMAX_FREE_SHARED_KEYS, 'MiniMax-Text-01': MINIMAX_FREE_SHARED_KEYS[-1:]
}
MINIMAX_SCNET_FREE_API_V1_KEYS = {
    'MiniMax-Text-01-456B': '8', 'MiniMax-M1-456B': '10', 
}


'''MiniMaxEndpoints'''
class MiniMaxEndpoints(BaseEndpoint):
    provider, model = "minimaxi", "minimaxi"
    def __init__(self, **kwargs):
        super(MiniMaxEndpoints, self).__init__(**kwargs)
        TI2T = ModelIOType.create(inputs=[Modality.TEXT, Modality.IMAGE], outputs=[Modality.TEXT])
        for ver in list(MINIMAX_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")] if ver in ['MiniMax-M2', 'MiniMax-M2-Stable'] else [ModelIOType.fromtag("T2T"), TI2T])
            self.registerapi(
                version=ver, name="officialapiv1", io=ModelIOType.fromtag("T2T"), handler="officialapiv1",
                priority=100, note='official api: https://api.minimaxi.com/v1'
            )
            if ver not in ['MiniMax-M2', 'MiniMax-M2-Stable']: self.registerapi(
                version=ver, name="visionofficialapiv1", io=TI2T, handler="visionofficialapiv1", priority=100, note='official api: https://api.minimaxi.com/v1'
            )
        for ver in list(MINIMAX_SCNET_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="scnetapiv1", io=ModelIOType.fromtag("T2T"), handler="scnetapiv1",
                priority=10, note='thirdpart api: https://www.scnet.cn/acx/chatbot/v1/chat/completion'
            )
    '''officialapiv1'''
    def officialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://api.minimaxi.com/v1', candidate_api_keys=MINIMAX_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''visionofficialapiv1'''
    def visionofficialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.visionopenaisdk(
            base_url='https://api.minimaxi.com/v1', candidate_api_keys=MINIMAX_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''scnetapiv1'''
    def scnetapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        # init
        request_overrides, cookies = request_overrides or {}, {"Token": secrets.token_hex(16)}
        headers = {
            "accept": "text/event-stream", "content-type": "application/json", "referer": "https://www.scnet.cn/", "origin": "https://www.scnet.cn",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Safari/537.36 Edg/135.0.0.0",
        }
        # _scnetextractor
        def _scnetextractor(chunk: Union[str, Dict[str, Any]]) -> Optional[str]:
            if isinstance(chunk, dict): return chunk.get("content")
            return None
        # payload
        payload = {
            "conversationId": "", "content": "", "thinkingEnable": False, "onlineEnable": False, "modelId": "", "textFile": [], "imageFile": [], "clusterId": ""
        }
        for key in list(payload.keys()):
            if key in req.extra_payload: payload[key] = req.extra_payload[key]
        payload['content'] = req.text
        payload['modelId'] = MINIMAX_SCNET_FREE_API_V1_KEYS[version]
        # post request
        resp = self.session.post("https://www.scnet.cn/acx/chatbot/v1/chat/completion", headers=headers, json=payload, stream=True, cookies=cookies, **request_overrides)
        resp.raise_for_status()
        # parse results
        processed_stream = StreamSanitizer(
            intro_value="data:", to_json=True, skip_markers=["[done]", "[DONE]"], content_extractor=_scnetextractor, yield_raw_on_error=False
        ).iter(data=resp.iter_content(chunk_size=None))
        streaming_text = ""
        for content_chunk in processed_stream:
            if content_chunk and isinstance(content_chunk, str):
                streaming_text += content_chunk
        streaming_text = streaming_text.strip().removesuffix('[done]').removesuffix('[DONE]')
        # return
        return ChatResponse(text=streaming_text, raw=resp)


'''
Function:
    Implementation of ByteDanceDoubaoEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import random
from .base import BaseEndpoint
from ..utils import ChatRequest, ChatResponse, ModelIOType, Modality


'''SETTINGS'''
DOUBAO_ARK_FREE_API_V3_KEYS = {
    'doubao': [
        {'version': 'ep-20241224194233-9mshp', 'key': 'CP8yLN3fMBkBTKbkBbQ-BeGJFREg6Gmn4F9S-UdamYoYTV_KvmGXkBkQKk8HOLKJ62KK6UkuFqVfc2oSccFqmnQt3HHCquDSEOgLUA=='},
        {'version': 'ep-20250102142238-88hxn', 'key': 'dUfnths4KAdt6nf9GfqpkCUIX_dFvvK50p25ycKw-k0KvY9xdcIMDbbAP9214WVHHPoAEGOCNC-zG3Eo9R-uf-_hKYOQzVKgMsToJQ=='},
    ],
    'doubao-seed-1-6-251015': [
        {'version': 'doubao-seed-1-6-251015', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
    'doubao-1-5-pro-32k-250115': [
        {'version': 'doubao-1-5-pro-32k-250115', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
        {'version': 'doubao-1-5-pro-32k-250115', 'key': 'Fu2qVXf5NkhPn4ix9464ktl3HZryIyhQbioFk4_YwWcdaFWykFraXC6P5cPXa85v6elNOnJ-JUk_GptnQVuuEmRs94YNPCQLLDAeSA=='},
    ],
    'doubao-1-5-pro-32k-character-250715': [
        {'version': 'doubao-1-5-pro-32k-character-250715', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
    'doubao-1-5-pro-32k-character-250228': [
        {'version': 'doubao-1-5-pro-32k-character-250228', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
    'doubao-1-5-lite-32k-250115': [
        {'version': 'doubao-1-5-lite-32k-250115', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
    'doubao-seed-1-6-flash-250615': [
        {'version': 'doubao-seed-1-6-flash-250615', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
    'doubao-seed-1-6-flash-250715': [
        {'version': 'doubao-seed-1-6-flash-250715', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
    'doubao-seed-coder-250915': [
        {'version': 'doubao-seed-coder-250915', 'key': 'Ax08S1LZC-miCjDuWPTJ5MDI3EQs0PxY1udtOYFMr4lXfj1w0vibymwBXZiiu2W7kA1hpcRfJLt-0oSREM808nIVlJpAROL5xWJRFQ=='},
    ],
}


'''ByteDanceDoubaoEndpoints'''
class ByteDanceDoubaoEndpoints(BaseEndpoint):
    provider, model = "bytedance", "doubao"
    def __init__(self, **kwargs):
        super(ByteDanceDoubaoEndpoints, self).__init__(**kwargs)
        TI2T = ModelIOType.create(inputs=[Modality.TEXT, Modality.IMAGE], outputs=[Modality.TEXT])
        for ver in list(DOUBAO_ARK_FREE_API_V3_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="arkapiv3", io=ModelIOType.fromtag("T2T"), handler="arkapiv3",
                priority=10, note='thirdpart api: https://ark.cn-beijing.volces.com/api/v3'
            )
    '''arkapiv3'''
    def arkapiv3(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        vk = random.choice(DOUBAO_ARK_FREE_API_V3_KEYS[version])
        return self.openaisdk(
            base_url='https://ark.cn-beijing.volces.com/api/v3', candidate_api_keys={vk['version']: [vk['key']]}, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=vk['version']
        )


'''initialize'''
from .common import (
    ModelScopeEndpoints,
)
from .utils import (
    ModelIOType, Modality, ChatRequest, ChatResponse, APISpec, ModelVariant, SecretUtils, StreamSanitizer, resp2json
)
from .providers import (
    BaseEndpoint, EndpointRegistry, IFLYTEKSparkEndpoints, BaiduQianfanEndpoints, OpenAIChatGPTEndpoints, ZhipuGLMEndpoints,
    HighflyerDeepSeekEndpoints, ByteDanceDoubaoEndpoints, AlibabaQwenEndpoints, MiniMaxEndpoints,
)


'''
Function:
    Implementation of Registered Models
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from .gpthub import (
    EndpointRegistry, ModelScopeEndpoints, IFLYTEKSparkEndpoints, BaiduQianfanEndpoints, OpenAIChatGPTEndpoints, ZhipuGLMEndpoints,
    HighflyerDeepSeekEndpoints, ByteDanceDoubaoEndpoints, AlibabaQwenEndpoints, MiniMaxEndpoints,
)


'''REGISTERED_ENDPOINTS'''
REGISTERED_ENDPOINTS = EndpointRegistry()
REGISTERED_ENDPOINTS.register(ModelScopeEndpoints)
REGISTERED_ENDPOINTS.register(IFLYTEKSparkEndpoints)
REGISTERED_ENDPOINTS.register(BaiduQianfanEndpoints)
REGISTERED_ENDPOINTS.register(OpenAIChatGPTEndpoints)
REGISTERED_ENDPOINTS.register(ZhipuGLMEndpoints)
REGISTERED_ENDPOINTS.register(HighflyerDeepSeekEndpoints)
REGISTERED_ENDPOINTS.register(ByteDanceDoubaoEndpoints)
REGISTERED_ENDPOINTS.register(AlibabaQwenEndpoints)
REGISTERED_ENDPOINTS.register(MiniMaxEndpoints)


'''initialize'''
from .zhipu import ZhipuGLMEndpoints
from .minimax import MiniMaxEndpoints
from .oai import OpenAIChatGPTEndpoints
from .baidu import BaiduQianfanEndpoints
from .alibaba import AlibabaQwenEndpoints
from .iflytek import IFLYTEKSparkEndpoints
from .bytedance import ByteDanceDoubaoEndpoints
from .base import BaseEndpoint, EndpointRegistry
from .highflyer import HighflyerDeepSeekEndpoints


'''
Function:
    Implementation of HighflyerDeepSeekEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from .base import BaseEndpoint
from ..utils import ChatRequest, ChatResponse, ModelIOType


'''SETTINGS'''
DEEPSEEK_FREE_API_KEYS = {
    'deepseek-chat': [
        'iWSTxC-YSd4nupox9dX6Y4DeuEYHCvUrHRpsRbDDuQ9Ek2wG0eH8VjK-QvmHVTXhlthPqcvAfAPfmSIjSlQJ4x-ydYEqQdyKYCCqjQ==',
        'ueXC8zJruW0b5dwPNgJuRpY37vBhky2lD94D_EhzisS-dDTRLm9hLnNQbmZz-_xcFi_tPVXW-SyNtwI1ALxEZmKZ83WWm6dkVA1hhg==',
    ],
}
DEEPSEEK_ALIYUNCS_FREE_SHARED_KEYS = [
    'eFCw8sVzThf5PlQAfGP6rEU-DWTz8h-OOUxuoPuzwMNHTcY9RwEljO87aqFXsa8v8NRZxIoNxJrWSLnipkMSQQLQi8dB85b7cpezFg==', 
    'L7XiMAoZGuVmbsWJ4mKoV7OySWFKl-ZM599F0-C_ZA3gQyj1TlB5L2fILq_l8hyonltIv53Bpk_xTWVJKWfRBmU3oyi0Msdr4ek-GQ==', 
    'VkS9va4uhcBBAMh8nJo0PhrLBl6H1BlLv4HQp1wpVZpYzz4qhhAH0ePOQaIJFEh7rgT3Y4pvgUnx5QrLmIN255sup3pcK5Tlk3l19Q==', 
    'DGT_SCWxaDnV3DouSk-TBKqg7sRVztPCuOtW2sgdAvKkk0vZzAeklOAj51YhH-2hKAhvHxM0DhpVBozafSdCrhiW-e3xsaBZ79Bu7g==', 
    'wZhgD2y9dYHO_xX2B1bil25qpl0w0pZ0sbyX9GhCDwamRVSMBJUeKYg4Ssd8YyZsp7xTshkUBOGr5KiGmXB4ZC6Fk87Ff6hV-yZONw==', 
    'oW1la7eBruvbYUSlO8q3tESc_FPGGoKKLxFXi12HfL1JVhF-c5ShfxNSY2qkKMsJwFI3ffzZ40QiYGOJqNgi2zagFyEKArhdGKRHeQ==', 
    'q50HVPVK5tBh0yq32a1-7xCxczVqyNIacwRPZy0OEUZYd4FDqW1B1ey9hAn4lVO3Rb7z8_nacbREU7eotkeFnQDhiWtFhP1UUS8Rag==',
    'zcgH7O1RorDk_HMt2Q00DeFxNuSVnM6DHS6QBGss9k3ptJnIkPXCkH2Dp3WyU4EsT9Bn_ogGmHmaFVVhuBFCuKHN2yeWiLWwvGN2QA==',
    'OMKEjdwNDmhSlpEz7dORBJ3nJhEcayS27f_o5brlI6qeNKzwmoIxGrzY0YwOHmiAE17fWAH-BOGkyl1UBl5SUbrN-flCorFt4_6pFw==',
]
DEEPSEEK_ALIYUNCS_FREE_API_V1_KEYS = {
    'deepseek-v3': DEEPSEEK_ALIYUNCS_FREE_SHARED_KEYS, 'deepseek-r1': DEEPSEEK_ALIYUNCS_FREE_SHARED_KEYS, 'deepseek-v3.1': DEEPSEEK_ALIYUNCS_FREE_SHARED_KEYS,
    'deepseek-v3.2': DEEPSEEK_ALIYUNCS_FREE_SHARED_KEYS, 'deepseek-v3.2-exp': DEEPSEEK_ALIYUNCS_FREE_SHARED_KEYS,
}
DEEPSEEK_QIANFAN_FREE_SHARED_KEYS = [
    'oiP1BpFuiPBfnyHJm9i5TAgwMZG3Y0Vd8UBqTmxaS2M7MNVHtz14eNsaOKa5n2T-2DDthiKYFZU25JAzlgUAECa4KIrn5s08Xete_El9TPTOTDPTvzsQaDpE-5JSrTYYrmPaWgmp7hYcfEIhYk9nsl62zhMcBLIk_7gXV9AXfb8=', 
    'DTeSqHsTdBZthSJfrAR575nGNh4ZGNk0Jz-hU5YrMZicCeJuxUJDO9YJACGcTD9UQ60f41eYwv3Gl6okp7hT3LYHTClIApx_nsOSVty2_D-BViS9qTqkQuh-ZZZSZOfWrBtglkl7I03ZQW6o1bug3uq75HEjtBMeK-dWtQJTlvE=', 
    'mHXHSnXsn562akeQfJDEvheYzdMtzVXuhLh-I8_NRjn5_yB0R8zGZpWaXM7rlN8SbL-kneABpK5FsMWWMm2X0ClHX3qqBjFGALf6EXzcpuIj6H2wcyklxHmn0i2AZnaGpxffSKuyoARrgY0lmokpMI0o_1OoxHbIeykhoyCW7R8='
]
DEEPSEEK_QIANFAN_FREE_API_V2_KEYS = {'deepseek-v3': DEEPSEEK_QIANFAN_FREE_SHARED_KEYS}


'''HighflyerDeepSeekEndpoints'''
class HighflyerDeepSeekEndpoints(BaseEndpoint):
    provider, model = "high-flyer", "deepseek"
    def __init__(self, **kwargs):
        super(HighflyerDeepSeekEndpoints, self).__init__(**kwargs)
        for ver in list(DEEPSEEK_FREE_API_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="officialapi", io=ModelIOType.fromtag("T2T"), handler="officialapi",
                priority=100, note='official api: https://api.deepseek.com'
            )
        for ver in list(DEEPSEEK_ALIYUNCS_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="aliyuncsapiv1", io=ModelIOType.fromtag("T2T"), handler="aliyuncsapiv1",
                priority=10, note='thirdpart api: https://dashscope.aliyuncs.com/compatible-mode/v1'
            )
        for ver in list(DEEPSEEK_QIANFAN_FREE_API_V2_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="qianfanapiv2", io=ModelIOType.fromtag("T2T"), handler="qianfanapiv2",
                priority=10, note='thirdpart api: https://qianfan.baidubce.com/v2'
            )
    '''officialapi'''
    def officialapi(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://api.deepseek.com', candidate_api_keys=DEEPSEEK_FREE_API_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''aliyuncsapiv1'''
    def aliyuncsapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://dashscope.aliyuncs.com/compatible-mode/v1', candidate_api_keys=DEEPSEEK_ALIYUNCS_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''qianfanapiv2'''
    def qianfanapiv2(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://qianfan.baidubce.com/v2', candidate_api_keys=DEEPSEEK_QIANFAN_FREE_API_V2_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )


'''initialize'''
from .misc import resp2json
from .secret import SecretUtils
from .webstream import StreamSanitizer
from .models import ModelIOType, Modality, ChatRequest, ChatResponse, APISpec, ModelVariant


'''
Function:
    Implementation of ZhipuGLMEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from .base import BaseEndpoint
from ..utils import ChatRequest, ChatResponse, ModelIOType


'''SETTINGS'''
GLM_FREE_API_V4_KEYS = {
    'glm-4-flash': [
        '41FBx1_ZwnT1-60C4MqbIkC7JZDBjbWEeoIYF_Oiiqct6D76bN5UUNSE3WjBWiC6RK0WGqU18-gu2ibOsHqoh1WEhzfXbMrmDlz8jpjTWcpsNvPXT8HeP4uVhuielvFI', 
        'g48qv0w9UpBhhhS4_4Pll844fO5SgHKSQStWnjzrzaveW-uwhMUWEak7LhRs-zMVJbXNPB2m3gsvSTnSd-cos8qk1yMV0X_i77yUqgPWN9wDGXgysETY5q09DtDKY_AR', 
        'iJguE3LqmI0AVxD6jqa0reYaHzUz2c7WMnKNcT0MONVuuuIobBJ5pqVC9Eu6Mu2iCJ-guc2b-6dmUNytDjW2PKiNW48pf920newsQtFoPCTUrklhakq-tz7jyvFQ7Hol', 
        '2WgmiDRi7g54Yyk_YPxv0i0MJFDOlyMrPfFwuYAK4YvVU774Zcx5BtAmr2JTkygYp7ZfCprP6tp1l2w6QpOGReB9ATRLIbsRELr36GVImVrJqGtV5csNMBO4OqhEp5eT', 
        'O168BowcawcXE-lUKVJx2bX4pwffPG0zp23jWolVd-IbhdN4XP8LAb6DyKbu_GZTxKPPkpeuCRtE-jRwzTh5KgGCMR4PUkMHkwtY-XlepIxlHhFEp8Jrgni_KnvEsEs2', 
        'BRDl_p4-kbu7I8o9DQi_TZodSuRb_ytpZEKGlRbneF8VNWGIjgU9WwsnN9WfSmo484ryXlnzDJS3tuZ4jtWQNkitsoZZ7OUaWA646Nueszd4PXUeij1r6wDaWS0YdFRh', 
        'tt8c_3Y1gEVwjLa6eSgA4-cjI1TWQTHHFhLo2ju7l4UTXkZghQVFwff-Wf2-en6pM6hZcJ4T28x0fXEkGDEdVRGHzcU-SsnDNK9wu0oLROmt6_7L3vOz3kOdVkX0kbQY', 
        'KGoKIMwCt-fzCeRsGaR56HYg6RnDeAppnQKL0ygCz55-GWC2uemKz727siMKH5B5jRv-Dg5siTmO7nx-MS2wW-D50--V-uLnp3tHhoDDI8ta1C9yUWy1Q56RgHOcGNes', 
        'n9IzbV6URId290jgrTdPbRurYt2kDO3RDQzgX-B_tbHUNqbGpNoBHqxHQiz8gDiZgPWekA07kp7l1aywVGawGQlf7AyUkOjV43s6vG8pT7blhgox488Y6Zrhsg-e6wj-', 
        'HFFuMzZ9mE7MUGD_ysP6ZiEPkANBN2iRyAapZYY08VdyAlKd327GWtZUEG6lqZBHjlfCYCtQrACmDqK8GrMYm2JKbiYe7hwtDYQUizTnnP_GyWT2dAf8w2rfHAvfnUSK', 
        'gAYd6e3XCqJ0tEIIFiPxSM7q_VCD22E3Hjl9alLNwHEvLtjyxJ0x4wjsd54psEPfo1hCnIKTbCmfybsN0OrYBnOg54nsKiGyShjq6zyBkXyHHq3lFRM4gDnXIuXryuXq',
    ], 
    'glm-4.5-flash': [
        '41FBx1_ZwnT1-60C4MqbIkC7JZDBjbWEeoIYF_Oiiqct6D76bN5UUNSE3WjBWiC6RK0WGqU18-gu2ibOsHqoh1WEhzfXbMrmDlz8jpjTWcpsNvPXT8HeP4uVhuielvFI', 
        'g48qv0w9UpBhhhS4_4Pll844fO5SgHKSQStWnjzrzaveW-uwhMUWEak7LhRs-zMVJbXNPB2m3gsvSTnSd-cos8qk1yMV0X_i77yUqgPWN9wDGXgysETY5q09DtDKY_AR', 
        'iJguE3LqmI0AVxD6jqa0reYaHzUz2c7WMnKNcT0MONVuuuIobBJ5pqVC9Eu6Mu2iCJ-guc2b-6dmUNytDjW2PKiNW48pf920newsQtFoPCTUrklhakq-tz7jyvFQ7Hol', 
        '2WgmiDRi7g54Yyk_YPxv0i0MJFDOlyMrPfFwuYAK4YvVU774Zcx5BtAmr2JTkygYp7ZfCprP6tp1l2w6QpOGReB9ATRLIbsRELr36GVImVrJqGtV5csNMBO4OqhEp5eT', 
        'O168BowcawcXE-lUKVJx2bX4pwffPG0zp23jWolVd-IbhdN4XP8LAb6DyKbu_GZTxKPPkpeuCRtE-jRwzTh5KgGCMR4PUkMHkwtY-XlepIxlHhFEp8Jrgni_KnvEsEs2', 
        'BRDl_p4-kbu7I8o9DQi_TZodSuRb_ytpZEKGlRbneF8VNWGIjgU9WwsnN9WfSmo484ryXlnzDJS3tuZ4jtWQNkitsoZZ7OUaWA646Nueszd4PXUeij1r6wDaWS0YdFRh', 
        'tt8c_3Y1gEVwjLa6eSgA4-cjI1TWQTHHFhLo2ju7l4UTXkZghQVFwff-Wf2-en6pM6hZcJ4T28x0fXEkGDEdVRGHzcU-SsnDNK9wu0oLROmt6_7L3vOz3kOdVkX0kbQY', 
        'KGoKIMwCt-fzCeRsGaR56HYg6RnDeAppnQKL0ygCz55-GWC2uemKz727siMKH5B5jRv-Dg5siTmO7nx-MS2wW-D50--V-uLnp3tHhoDDI8ta1C9yUWy1Q56RgHOcGNes', 
        'n9IzbV6URId290jgrTdPbRurYt2kDO3RDQzgX-B_tbHUNqbGpNoBHqxHQiz8gDiZgPWekA07kp7l1aywVGawGQlf7AyUkOjV43s6vG8pT7blhgox488Y6Zrhsg-e6wj-', 
        'HFFuMzZ9mE7MUGD_ysP6ZiEPkANBN2iRyAapZYY08VdyAlKd327GWtZUEG6lqZBHjlfCYCtQrACmDqK8GrMYm2JKbiYe7hwtDYQUizTnnP_GyWT2dAf8w2rfHAvfnUSK', 
        'gAYd6e3XCqJ0tEIIFiPxSM7q_VCD22E3Hjl9alLNwHEvLtjyxJ0x4wjsd54psEPfo1hCnIKTbCmfybsN0OrYBnOg54nsKiGyShjq6zyBkXyHHq3lFRM4gDnXIuXryuXq',
    ],
}


'''ZhipuGLMEndpoints'''
class ZhipuGLMEndpoints(BaseEndpoint):
    provider, model = "zhipu", "glm"
    def __init__(self, **kwargs):
        super(ZhipuGLMEndpoints, self).__init__(**kwargs)
        for ver in list(GLM_FREE_API_V4_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="officialapiv4", io=ModelIOType.fromtag("T2T"), handler="officialapiv4",
                priority=100, note='official api: https://open.bigmodel.cn/api/paas/v4/'
            )
    '''officialapiv4'''
    def officialapiv4(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://open.bigmodel.cn/api/paas/v4/', candidate_api_keys=GLM_FREE_API_V4_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )


'''
Function:
    Implementation of Data Models
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from __future__ import annotations
from enum import Enum
from dataclasses import dataclass, field
from typing import Any, Dict, FrozenSet, Iterable, Optional, Tuple, Union


'''ModelIOType'''
@dataclass(frozen=True, slots=True)
class ModelIOType:
    class Modality(Enum):
        TEXT, IMAGE, AUDIO, VIDEO = "text", "image", "audio", "video"
        @property
        def short(self) -> str:
            return {self.TEXT: "T", self.IMAGE: "I", self.AUDIO: "A", self.VIDEO: "V"}[self]
        @classmethod
        def fromany(cls, x: Union["ModelIOType.Modality", str]) -> "ModelIOType.Modality":
            if isinstance(x, cls): return x
            if not isinstance(x, str): raise TypeError(f"Unsupported modality type: {type(x)}")
            s = x.strip().lower()
            alias = {
                "t": "text", "text": "text", "txt": "text", "i": "image", "img": "image", "image": "image",
                "a": "audio", "aud": "audio", "audio": "audio", "v": "video", "vid": "video", "video": "video",
            }
            if s not in alias: raise ValueError(f"Unknown modality string: {x!r}")
            return cls(alias[s])
    inputs: FrozenSet[Modality]
    outputs: FrozenSet[Modality]
    name: Optional[str] = field(default=None, compare=False)
    meta: FrozenSet[str] = field(default_factory=frozenset, compare=False)
    @property
    def is_multimodal_input(self) -> bool:
        return len(self.inputs) > 1
    @property
    def is_multimodal_output(self) -> bool:
        return len(self.outputs) > 1
    @property
    def tag(self) -> str:
        def side(mods: FrozenSet[ModelIOType.Modality]) -> str:
            if len(mods) == 1: return next(iter(mods)).short
            return "MM"
        return f"{side(self.inputs)}2{side(self.outputs)}"
    '''postinit'''
    def __post_init__(self):
        if not self.inputs: raise ValueError("inputs cannot be empty")
        if not self.outputs: raise ValueError("outputs cannot be empty")
    '''tofrozenset'''
    @staticmethod
    def tofrozenset(mods: Iterable[Union["ModelIOType.Modality", str]]) -> FrozenSet["ModelIOType.Modality"]:
        return frozenset(ModelIOType.Modality.fromany(m) for m in mods)
    '''create'''
    @classmethod
    def create(cls, inputs: Iterable[Union["ModelIOType.Modality", str]], outputs: Iterable[Union["ModelIOType.Modality", str]], *, name: Optional[str] = None, meta: Optional[Iterable[str]] = None) -> "ModelIOType":
        return cls(inputs=cls.tofrozenset(inputs), outputs=cls.tofrozenset(outputs), name=name, meta=frozenset(meta or ()))
    '''detailedtag'''
    def detailedtag(self) -> str:
        def side(mods: FrozenSet[ModelIOType.Modality]) -> str:
            return "".join(sorted((m.short for m in mods)))
        return f"{side(self.inputs)}2{side(self.outputs)}"
    '''fromtag'''
    @classmethod
    def fromtag(cls, tag: str, *, name: Optional[str] = None) -> "ModelIOType":
        s = tag.strip().upper().replace(" ", "")
        if "2" not in s: raise ValueError(f"Invalid tag: {tag!r}")
        left, right = s.split("2", 1)
        if left == "MM" or right == "MM": raise ValueError("Cannot parse 'MM' tag because it does not specify exact modalities. Use create(inputs=..., outputs=...) instead.")
        short2mod = {"T": cls.Modality.TEXT, "I": cls.Modality.IMAGE, "A": cls.Modality.AUDIO, "V": cls.Modality.VIDEO}
        def _parseside(side: str) -> FrozenSet[ModelIOType.Modality]:
            if not side: raise ValueError(f"Empty side in tag: {tag!r}")
            mods = []
            for ch in side:
                if ch not in short2mod: raise ValueError(f"Unknown modality short '{ch}' in tag: {tag!r}")
                mods.append(short2mod[ch])
            return frozenset(mods)
        return cls(inputs=_parseside(left), outputs=_parseside(right), name=name)
    '''str'''
    def __str__(self) -> str:
        n = f"{self.name}: " if self.name else ""
        return f"{n}{self.detailedtag()} (canonical={self.tag})"


'''Modality'''
Modality = ModelIOType.Modality


'''ChatRequest'''
@dataclass(slots=True)
class ChatRequest:
    text: str = ""
    images: Tuple[Any, ...] = ()
    openai: Dict[str, Any] = field(default_factory=lambda: dict({'client': {}, 'client.responses.create': {}, 'client.chat.completions.create': {}}))
    websocket: Dict[str, Any] = field(default_factory=lambda: dict({'client': {}}))
    extra_payload: Dict[str, Any] = field(default_factory=dict)
    meta: Dict[str, Any] = field(default_factory=dict)


'''ChatResponse'''
@dataclass(slots=True)
class ChatResponse:
    text: str
    raw: Any = None
    api_used: str = ""
    variant_used: str = ""


'''APISpec'''
@dataclass(frozen=True, slots=True)
class APISpec:
    name: str
    io: ModelIOType
    handler: str
    priority: int = 0
    note: str = ""


'''ModelVariant'''
@dataclass(slots=True)
class ModelVariant:
    provider: str
    model: str
    version: str
    io_supported: FrozenSet[ModelIOType]
    apis: Dict[str, APISpec] = field(default_factory=dict)
    @property
    def id(self) -> str:
        return f"{self.provider}:{self.model}:{self.version}"


'''
Function:
    Implementation of StreamSanitizer
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import re
import sys
import json
import codecs
import asyncio
import functools
from itertools import chain
from dataclasses import dataclass, field
from typing import (
    Any, AsyncGenerator, AsyncIterable, Callable, Dict, Generator, Iterable, List, Literal, Optional, Union
)


'''settings'''
EncodingType = Literal[
    "utf-8", "utf-16", "utf-32", "ascii", "latin1", "cp1252", "iso-8859-1", "iso-8859-2", "windows-1250",
    "windows-1251", "windows-1252", "gbk", "big5", "shift_jis", "euc-jp", "euc-kr",
]


'''compileregexes'''
def compileregexes(patterns: Optional[List[Union[str, re.Pattern[str]]]]) -> Optional[List[re.Pattern[str]]]:
    if not patterns: return None
    out: List[re.Pattern[str]] = []
    for i, p in enumerate(patterns):
        try:
            if isinstance(p, str): out.append(re.compile(p))
            elif isinstance(p, re.Pattern): out.append(p)
            else: raise ValueError(f"Pattern at index {i} must be a string or compiled regex pattern, got {type(p).__name__}")
        except re.error as e:
            raise ValueError(f"Invalid regex pattern at index {i}: '{p}' - {e}")
    return out


'''processchunk'''
def processchunk(chunk: str, intro_value: str, to_json: bool, skip_markers: List[str], strip_chars: Optional[str], yield_raw_on_error: bool, 
                 error_handler: Optional[Callable[[Exception, str], Optional[Any]]] = None, skip_regexes: Optional[List[re.Pattern[str]]] = None, 
                 extract_regexes: Optional[List[re.Pattern[str]]] = None) -> Union[str, Dict[str, Any], Any, None]:
    if not isinstance(chunk, str) or not chunk: return None
    sanitized_chunk = chunk
    if intro_value and len(chunk) >= len(intro_value) and chunk[: len(intro_value)] == intro_value: sanitized_chunk = chunk[len(intro_value) :]
    if strip_chars is not None: sanitized_chunk = sanitized_chunk.strip(strip_chars)
    else: sanitized_chunk = sanitized_chunk.lstrip()
    if not sanitized_chunk or any(marker == sanitized_chunk for marker in skip_markers): return None
    if extract_regexes:
        extracted_content = None
        for rx in extract_regexes:
            m = rx.search(sanitized_chunk)
            if not m: continue
            if m.groups(): extracted_content = m.group(1) if len(m.groups()) == 1 else str(m.groups())
            else: extracted_content = m.group(0)
            break
        if extracted_content is None:
            if not to_json: return None
        else:
            sanitized_chunk = extracted_content
    if skip_regexes and any(rx.search(sanitized_chunk) for rx in skip_regexes): return None
    if to_json:
        try:
            if (len(sanitized_chunk) >= 2 and sanitized_chunk[0] not in "{[" and sanitized_chunk[-1] not in "}]"): sanitized_chunk = sanitized_chunk.strip()
            return json.loads(sanitized_chunk)
        except Exception as e:
            if error_handler:
                try:
                    handled = error_handler(e, sanitized_chunk)
                    if handled is not None: return handled
                except Exception:
                    pass
            return sanitized_chunk if yield_raw_on_error else None
    return sanitized_chunk


'''decodebytessync'''
def decodebytessync(byte_iter: Iterable[bytes], encoding: EncodingType, errors: str, buffer_size: int) -> Generator[str, None, None]:
    try: decoder = codecs.getincrementaldecoder(encoding)(errors=errors)
    except LookupError: decoder = codecs.getincrementaldecoder("utf-8")(errors=errors)
    buffer = bytearray(buffer_size)
    view = memoryview(buffer)
    for b in byte_iter:
        if not b: continue
        try:
            if len(b) <= buffer_size:
                buffer[: len(b)] = b
                text = decoder.decode(view[: len(b)], final=False)
            else: text = decoder.decode(b, final=False)
            if text: yield text
        except UnicodeDecodeError: yield f"[Encoding Error: Could not decode bytes with {encoding}]\n"
    try:
        tail = decoder.decode(b"", final=True)
        if tail: yield tail
    except UnicodeDecodeError:
        yield f"[Encoding Error: Could not decode final bytes with {encoding}]\n"


'''decodebytesasync'''
async def decodebytesasync(byte_aiter: AsyncIterable[bytes], encoding: EncodingType, errors: str, buffer_size: int) -> AsyncGenerator[str, None]:
    try: decoder = codecs.getincrementaldecoder(encoding)(errors=errors)
    except LookupError: decoder = codecs.getincrementaldecoder("utf-8")(errors=errors)
    buffer = bytearray(buffer_size)
    view = memoryview(buffer)
    async for b in byte_aiter:
        if not b: continue
        try:
            if len(b) <= buffer_size:
                buffer[: len(b)] = b
                text = decoder.decode(view[: len(b)], final=False)
            else:
                text = decoder.decode(b, final=False)
            if text: yield text
        except UnicodeDecodeError: yield f"[Encoding Error: Could not decode bytes with {encoding}]\n"
    try:
        tail = decoder.decode(b"", final=True)
        if tail: yield tail
    except UnicodeDecodeError: yield f"[Encoding Error: Could not decode final bytes with {encoding}]\n"


'''splitcompletelines'''
def splitcompletelines(buf: str, line_delimiter: Optional[str]) -> tuple[list[str], str]:
    if not buf: return [], ""
    if line_delimiter is not None:
        parts = buf.split(line_delimiter)
        if len(parts) == 1: return [], buf
        return parts[:-1], parts[-1]
    parts_ke = buf.splitlines(True)
    if not parts_ke: return [], buf
    complete: list[str] = []
    remainder = ""
    for i, p in enumerate(parts_ke):
        if p.endswith("\n") or p.endswith("\r"):
            complete.append(p.rstrip("\r\n"))
        else:
            if i == len(parts_ke) - 1: remainder = p
            else: complete.append(p)
    return complete, remainder


'''StreamSanitizer'''
@dataclass(slots=True)
class StreamSanitizer:
    intro_value: str = "data:"
    to_json: bool = True
    skip_markers: Optional[List[str]] = None
    strip_chars: Optional[str] = None
    start_marker: Optional[str] = None
    end_marker: Optional[str] = None
    content_extractor: Optional[Callable[[Union[str, Dict[str, Any]]], Optional[Any]]] = None
    yield_raw_on_error: bool = True
    encoding: EncodingType = "utf-8"
    encoding_errors: str = "replace"
    buffer_size: int = 8192
    line_delimiter: Optional[str] = None
    error_handler: Optional[Callable[[Exception, str], Optional[Any]]] = None
    skip_regexes: Optional[List[Union[str, re.Pattern[str]]]] = None
    extract_regexes: Optional[List[Union[str, re.Pattern[str]]]] = None
    object_mode: Literal["as_is", "json", "str"] = "json"
    raw: bool = False
    output_formatter: Optional[Callable[[Any], Any]] = None
    debug: bool = False
    _skip_markers_eff: List[str] = field(init=False, repr=False)
    _skip_rx: Optional[List[re.Pattern[str]]] = field(init=False, repr=False, default=None)
    _extract_rx: Optional[List[re.Pattern[str]]] = field(init=False, repr=False, default=None)
    '''postinit'''
    def __post_init__(self) -> None:
        self._skip_markers_eff: List[str] = self.skip_markers or []
        self._skip_rx: Optional[List[re.Pattern[str]]] = compileregexes(self.skip_regexes)
        self._extract_rx: Optional[List[re.Pattern[str]]] = compileregexes(self.extract_regexes)
    '''_fmt'''
    def _fmt(self, item: Any) -> Any:
        return self.output_formatter(item) if self.output_formatter else item
    '''_rawsync'''
    def _rawsync(self, data: Any) -> Generator[Any, None, None]:
        if isinstance(data, (bytes, bytearray)):
            yield data.decode(self.encoding, self.encoding_errors)
            return
        if isinstance(data, str):
            yield data
            return
        if hasattr(data, "__iter__"):
            for x in data:
                if x is None: continue
                if isinstance(x, (bytes, bytearray)): yield x.decode(self.encoding, self.encoding_errors)
                else: yield x
            return
        if data is not None: yield data
    '''_rawasync'''
    async def _rawasync(self, data: Any) -> AsyncGenerator[Any, None]:
        if isinstance(data, (bytes, bytearray)):
            yield data.decode(self.encoding, self.encoding_errors)
            return
        if isinstance(data, str):
            yield data
            return
        if hasattr(data, "__aiter__"):
            async for x in data:
                if x is None: continue
                if isinstance(x, (bytes, bytearray)): yield x.decode(self.encoding, self.encoding_errors)
                else: yield x
            return
        if hasattr(data, "__iter__"):
            for x in data:
                if x is None: continue
                if isinstance(x, (bytes, bytearray)): yield x.decode(self.encoding, self.encoding_errors)
                else: yield x
            return
        if data is not None: yield data
    '''iter'''
    def iter(self, data: Any) -> Generator[Any, None, None]:
        if self.raw:
            yield from self._rawsync(data)
            return
        if data is None: return
        if isinstance(data, (dict, list, int, float, bool)):
            if self.object_mode == "as_is":
                yield self._fmt(data)
                return
            if self.object_mode == "str": data = str(data)
            else:
                try: data = json.dumps(data)
                except Exception: data = str(data)
        text_iter: Iterable[str]
        if isinstance(data, (bytes, bytearray)): text_iter = decodebytessync([bytes(data)], self.encoding, self.encoding_errors, self.buffer_size)
        elif isinstance(data, str): text_iter = [data]
        elif hasattr(data, "read") and callable(data.read):
            try:
                payload = data.read()
                if isinstance(payload, (bytes, bytearray)): text_iter = decodebytessync([bytes(payload)], self.encoding, self.encoding_errors, self.buffer_size)
                else: text_iter = [str(payload)]
            except Exception:
                text_iter = [str(data)]
        elif hasattr(data, "__iter__"):
            it = iter(data)
            first = next(it, None)
            if first is None: return
            stream = chain([first], it)
            if isinstance(first, (bytes, bytearray)): text_iter = decodebytessync((bytes(x) for x in stream), self.encoding, self.encoding_errors, self.buffer_size,)
            elif isinstance(first, str): text_iter = (x for x in stream if x is not None)
            else: text_iter = (str(x) for x in stream if x is not None)
        else:
            text_iter = [str(data)]
        buf, line_buf, found_start = "", "", self.start_marker is None
        end_keep = (len(self.end_marker) - 1) if self.end_marker else 0
        start_keep = (len(self.start_marker) - 1) if self.start_marker else 0
        start_keep = max(start_keep, 256) if self.start_marker else 0
        def _emitlinesfromtext(text: str) -> Generator[Any, None, None]:
            nonlocal line_buf
            if not text: return
            line_buf += text
            lines, line_buf = splitcompletelines(line_buf, self.line_delimiter)
            for ln in lines:
                use_extract = self._extract_rx if not self.content_extractor else None
                res = processchunk(ln, self.intro_value, self.to_json, self._skip_markers_eff, self.strip_chars, self.yield_raw_on_error, self.error_handler, self._skip_rx, use_extract)
                if res is None: continue
                if self.content_extractor:
                    try: final = self.content_extractor(res)
                    except Exception as e:
                        if self.debug: print(f"[StreamSanitizer] content_extractor error: {e}", file=sys.stderr)
                        continue
                    if final is None: continue
                    if self._extract_rx and isinstance(final, str):
                        extracted = None
                        for rx in self._extract_rx:
                            m = rx.search(final)
                            if not m: continue
                            if m.groups(): extracted = m.group(1) if len(m.groups()) == 1 else str(m.groups())
                            else: extracted = m.group(0)
                            break
                        if extracted is not None: yield self._fmt(extracted)
                    else: yield self._fmt(final)
                else:
                    yield self._fmt(res)
        try:
            for chunk in text_iter:
                if not chunk: continue
                buf += chunk
                if not found_start and self.start_marker:
                    idx = buf.find(self.start_marker)
                    if idx == -1:
                        buf = buf[-start_keep:] if start_keep else ""
                        continue
                    found_start = True
                    buf = buf[idx + len(self.start_marker) :]
                if found_start and self.end_marker:
                    idx = buf.find(self.end_marker)
                    if idx != -1:
                        active = buf[:idx]
                        buf = buf[idx + len(self.end_marker) :]
                        yield from _emitlinesfromtext(active)
                        if line_buf: yield from _emitlinesfromtext(self.line_delimiter or "\n")
                        found_start = self.start_marker is None
                        continue
                    if end_keep and len(buf) > end_keep:
                        active = buf[: len(buf) - end_keep]
                        buf = buf[len(buf) - end_keep :]
                        yield from _emitlinesfromtext(active)
                    else: continue
                else:
                    yield from _emitlinesfromtext(buf)
                    buf = ""
            if found_start:
                if self.end_marker: yield from _emitlinesfromtext(buf)
                else: yield from _emitlinesfromtext(buf)
                buf = ""
            if line_buf:
                rem = line_buf
                line_buf = ""
                for out in _emitlinesfromtext(rem + (self.line_delimiter or "\n")): yield out
        except Exception as e:
            if self.debug: print(f"[StreamSanitizer] stream error: {e}", file=sys.stderr)
    '''aiter'''
    async def aiter(self, data: Any) -> AsyncGenerator[Any, None]:
        if self.raw:
            async for x in self._rawasync(data): yield x
            return
        if data is None: return
        if isinstance(data, (dict, list, int, float, bool)):
            if self.object_mode == "as_is":
                yield self._fmt(data)
                return
            if self.object_mode == "str":
                data = str(data)
            else:
                try: data = json.dumps(data)
                except Exception: data = str(data)
        async def _as_text_aiter(obj: Any) -> AsyncGenerator[str, None]:
            if isinstance(obj, (bytes, bytearray)):
                async for t in decodebytesasync(_single_bytes_async(bytes(obj)), self.encoding, self.encoding_errors, self.buffer_size): yield t
                return
            if isinstance(obj, str):
                yield obj
                return
            if hasattr(obj, "read") and callable(obj.read):
                try:
                    payload = obj.read()
                    if isinstance(payload, (bytes, bytearray)):
                        async for t in decodebytesasync(_single_bytes_async(bytes(payload)), self.encoding, self.encoding_errors, self.buffer_size): yield t
                    else:
                        yield str(payload)
                    return
                except Exception:
                    yield str(obj)
                    return
            if hasattr(obj, "__aiter__"):
                it = obj.__aiter__()
                first = None
                async for first in it: break
                if first is None: return
                async def _chain(first_item, rest_it):
                    yield first_item
                    async for y in rest_it: yield y
                stream = _chain(first, it)
                if isinstance(first, (bytes, bytearray)):
                    async for t in decodebytesasync((bytes(x) async for x in stream), self.encoding, self.encoding_errors, self.buffer_size): yield t
                elif isinstance(first, str):
                    async for x in stream:
                        if x: yield x
                else:
                    async for x in stream:
                        if x is not None: yield str(x)
                return
            if hasattr(obj, "__iter__"):
                it = iter(obj)
                first = next(it, None)
                if first is None: return
                stream = chain([first], it)
                if isinstance(first, (bytes, bytearray)):
                    for t in decodebytessync((bytes(x) for x in stream), self.encoding, self.encoding_errors, self.buffer_size): yield t
                elif isinstance(first, str):
                    for x in stream:
                        if x: yield x
                else:
                    for x in stream:
                        if x is not None: yield str(x)
                return
            yield str(obj)
        async def _single_bytes_async(b: bytes) -> AsyncGenerator[bytes, None]: yield b
        buf, line_buf, found_start = "", "", self.start_marker is None
        end_keep = (len(self.end_marker) - 1) if self.end_marker else 0
        start_keep = (len(self.start_marker) - 1) if self.start_marker else 0
        start_keep = max(start_keep, 256) if self.start_marker else 0
        async def _emitlinesfromtext(text: str) -> AsyncGenerator[Any, None]:
            nonlocal line_buf
            if not text: return
            line_buf += text
            lines, line_buf2 = splitcompletelines(line_buf, self.line_delimiter)
            line_buf = line_buf2
            for ln in lines:
                use_extract = self._extract_rx if not self.content_extractor else None
                res = processchunk(ln, self.intro_value, self.to_json, self._skip_markers_eff, self.strip_chars, self.yield_raw_on_error, self.error_handler, self._skip_rx, use_extract)
                if res is None: continue
                if self.content_extractor:
                    try: final = self.content_extractor(res)
                    except Exception as e:
                        if self.debug: print(f"[StreamSanitizer] content_extractor error: {e}", file=sys.stderr)
                        continue
                    if final is None: continue
                    if self._extract_rx and isinstance(final, str):
                        extracted = None
                        for rx in self._extract_rx:
                            m = rx.search(final)
                            if not m: continue
                            if m.groups(): extracted = m.group(1) if len(m.groups()) == 1 else str(m.groups())
                            else: extracted = m.group(0)
                            break
                        if extracted is not None: yield self._fmt(extracted)
                    else:
                        yield self._fmt(final)
                else:
                    yield self._fmt(res)
        try:
            async for chunk in _as_text_aiter(data):
                if not chunk: continue
                buf += chunk
                if not found_start and self.start_marker:
                    idx = buf.find(self.start_marker)
                    if idx == -1:
                        buf = buf[-start_keep:] if start_keep else ""
                        continue
                    found_start = True
                    buf = buf[idx + len(self.start_marker) :]
                if found_start and self.end_marker:
                    idx = buf.find(self.end_marker)
                    if idx != -1:
                        active = buf[:idx]
                        buf = buf[idx + len(self.end_marker) :]
                        async for out in _emitlinesfromtext(active): yield out
                        if line_buf:
                            async for out in _emitlinesfromtext(self.line_delimiter or "\n"): yield out
                        found_start = self.start_marker is None
                        continue
                    if end_keep and len(buf) > end_keep:
                        active = buf[: len(buf) - end_keep]
                        buf = buf[len(buf) - end_keep :]
                        async for out in _emitlinesfromtext(active): yield out
                    else:
                        continue
                else:
                    async for out in _emitlinesfromtext(buf): yield out
                    buf = ""
            if found_start:
                async for out in _emitlinesfromtext(buf): yield out
            if line_buf:
                rem = line_buf
                line_buf = ""
                async for out in _emitlinesfromtext(rem + (self.line_delimiter or "\n")): yield out
        except Exception as e:
            if self.debug: print(f"[StreamSanitizer] async stream error: {e}", file=sys.stderr)
    '''call'''
    def __call__(self, data: Any) -> Union[Generator[Any, None, None], AsyncGenerator[Any, None]]:
        if hasattr(data, "__aiter__"): return self.aiter(data)
        return self.iter(data)
    '''decorator'''
    def decorator(self, func=None):
        def _wrap(f):
            if asyncio.iscoroutinefunction(f):
                @functools.wraps(f)
                async def aw(*args, **kwargs):
                    res = await f(*args, **kwargs)
                    return self(res)
                return aw
            else:
                @functools.wraps(f)
                def sw(*args, **kwargs):
                    res = f(*args, **kwargs)
                    return self(res)
                return sw
        return _wrap if func is None else _wrap(func)


'''
Function:
    Implementation of AlibabaQwenEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import secrets
from .base import BaseEndpoint
from typing import Union, Dict, Any, Optional
from ..utils import ChatRequest, ChatResponse, ModelIOType, Modality, StreamSanitizer


'''SETTINGS'''
QWEN_FREE_SHARED_KEYS = [
    'pG4WicNJC9wohzOPEg6kQcuag3xr5O9Uhe1VUf7J95Bvo9PhICkKqUR4jEJb83t3dpp6WYA7YRz3cW76o7Rns7ns9JDycckEBMpi-w==', 
    'LnML1ffAjOw0tInoedXkswF0nTGZQWh-Jnj_V_IBe05PSQcklAhNnjdLxcL0KTe--Sc85qRtQcfV7Zo-IdXwyrusJh-9njvTypGN5Q==', 
    'J6TXvPOvm7rMjGecgG3yta9OUSt7FRt2ao4bOaKMjlX2TMLiuSd0YXDiDWw-3k5WXRWAWh5z-s0xpo0DVhoyqnLWlCjujVe5Q0im2w==', 
    'AW9ZCVKq9Yycu2xlmrozwgZyXCKagE8MayrbedteSdLZ01Hhwl1yL9MTPUxxtfdpQr7m1QxvNY2j3jAc1i8UcEPGLyh0_lY9j31EwQ==', 
    'MyZsP9AJmtZeqxoxH1nV7LQ8zmikWJhc4hQfqf-XARa9Lmd9VJ04RikkPbv6WW_qzfWAxvbT_L1fc3wVBVbXT2o3McMCFxZL2n6mgw==', 
    '96q5jwtx1b-kNh6X-0m6OVB3yMIMIfQibrk3T7X6SGVlEJ_pHmKtB6CHq_inDKqg2MPRCoFtteqM3y1Y0RqX3Vcbeks5DPardB4JAw==', 
    '3RJVZBVB33R7elL8c7wFTSnwmnfSCwirGcUA1Xkku9xUtM4f78PNzCElAtjLhJ9ApVAh4JxjbVsdGMCcvvOepUT_1Q-Z1nmhfi4qQA==',
    'zcgH7O1RorDk_HMt2Q00DeFxNuSVnM6DHS6QBGss9k3ptJnIkPXCkH2Dp3WyU4EsT9Bn_ogGmHmaFVVhuBFCuKHN2yeWiLWwvGN2QA==',
    'OMKEjdwNDmhSlpEz7dORBJ3nJhEcayS27f_o5brlI6qeNKzwmoIxGrzY0YwOHmiAE17fWAH-BOGkyl1UBl5SUbrN-flCorFt4_6pFw==',
]
QWEN_FREE_API_V1_KEYS = {
    'qwen-plus': QWEN_FREE_SHARED_KEYS, 'qwen3-max': QWEN_FREE_SHARED_KEYS, 'qwen-flash': QWEN_FREE_SHARED_KEYS, 'qwen-turbo': QWEN_FREE_SHARED_KEYS,
    'qwen-long': QWEN_FREE_SHARED_KEYS, 'qwen3-omni-flash': QWEN_FREE_SHARED_KEYS, 'qwen-math-plus': QWEN_FREE_SHARED_KEYS, 'qwen-math-turbo': QWEN_FREE_SHARED_KEYS,
    'qwen3-coder-plus': QWEN_FREE_SHARED_KEYS, 'qwen3-coder-flash': QWEN_FREE_SHARED_KEYS, 'qwen-mt-plus': QWEN_FREE_SHARED_KEYS, 'qwen-mt-flash': QWEN_FREE_SHARED_KEYS,
    'qwen-mt-lite': QWEN_FREE_SHARED_KEYS, 'qwen-mt-turbo': QWEN_FREE_SHARED_KEYS, 'qwen-doc-turbo': QWEN_FREE_SHARED_KEYS, 'qwen-deep-research': QWEN_FREE_SHARED_KEYS,
    'qwen3-vl-plus': QWEN_FREE_SHARED_KEYS, 'qwen3-vl-flash': QWEN_FREE_SHARED_KEYS, 'qwen-vl-max': QWEN_FREE_SHARED_KEYS, 'qwen-vl-ocr': QWEN_FREE_SHARED_KEYS,
}
QWEN_SCNET_FREE_API_V1_KEYS = {
    'Qwen3-30B-A3B': '9', 'Qwen3-235B-A22B': '120', 
}


'''AlibabaQwenEndpoints'''
class AlibabaQwenEndpoints(BaseEndpoint):
    provider, model = "alibaba", "qwen"
    def __init__(self, **kwargs):
        super(AlibabaQwenEndpoints, self).__init__(**kwargs)
        TI2T = ModelIOType.create(inputs=[Modality.TEXT, Modality.IMAGE], outputs=[Modality.TEXT])
        for ver in list(QWEN_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")] if '-vl-' not in ver else [ModelIOType.fromtag("T2T"), TI2T])
            self.registerapi(
                version=ver, name="officialapiv1", io=ModelIOType.fromtag("T2T"), handler="officialapiv1",
                priority=100, note='official api: https://dashscope.aliyuncs.com/compatible-mode/v1'
            )
            if '-vl-' in ver: self.registerapi(
                version=ver, name="visionofficialapiv1", io=TI2T, handler="visionofficialapiv1", priority=100, note='official api: https://dashscope.aliyuncs.com/compatible-mode/v1'
            )
        for ver in list(QWEN_SCNET_FREE_API_V1_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="scnetapiv1", io=ModelIOType.fromtag("T2T"), handler="scnetapiv1",
                priority=10, note='thirdpart api: https://www.scnet.cn/acx/chatbot/v1/chat/completion'
            )
    '''officialapiv1'''
    def officialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://dashscope.aliyuncs.com/compatible-mode/v1', candidate_api_keys=QWEN_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''visionofficialapiv1'''
    def visionofficialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.visionopenaisdk(
            base_url='https://dashscope.aliyuncs.com/compatible-mode/v1', candidate_api_keys=QWEN_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''scnetapiv1'''
    def scnetapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        # init
        request_overrides, cookies = request_overrides or {}, {"Token": secrets.token_hex(16)}
        headers = {
            "accept": "text/event-stream", "content-type": "application/json", "referer": "https://www.scnet.cn/", "origin": "https://www.scnet.cn",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Safari/537.36 Edg/135.0.0.0",
        }
        # _scnetextractor
        def _scnetextractor(chunk: Union[str, Dict[str, Any]]) -> Optional[str]:
            if isinstance(chunk, dict): return chunk.get("content")
            return None
        # payload
        payload = {
            "conversationId": "", "content": "", "thinkingEnable": False, "onlineEnable": False, "modelId": "", "textFile": [], "imageFile": [], "clusterId": ""
        }
        for key in list(payload.keys()):
            if key in req.extra_payload: payload[key] = req.extra_payload[key]
        payload['content'] = req.text
        payload['modelId'] = QWEN_SCNET_FREE_API_V1_KEYS[version]
        # post request
        resp = self.session.post("https://www.scnet.cn/acx/chatbot/v1/chat/completion", headers=headers, json=payload, stream=True, cookies=cookies, **request_overrides)
        resp.raise_for_status()
        # parse results
        processed_stream = StreamSanitizer(
            intro_value="data:", to_json=True, skip_markers=["[done]", "[DONE]"], content_extractor=_scnetextractor, yield_raw_on_error=False
        ).iter(data=resp.iter_content(chunk_size=None))
        streaming_text = ""
        for content_chunk in processed_stream:
            if content_chunk and isinstance(content_chunk, str):
                streaming_text += content_chunk
        streaming_text = streaming_text.strip().removesuffix('[done]').removesuffix('[DONE]')
        # return
        return ChatResponse(text=streaming_text, raw=resp)


'''
Function:
    Implementation of IFLYTEKSparkEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import hmac
import json
import base64
import random
import hashlib
import websocket
from time import mktime
from datetime import datetime
from .base import BaseEndpoint
from urllib.parse import urlencode, urlparse
from wsgiref.handlers import format_date_time
from ..utils import resp2json, ChatRequest, ChatResponse, ModelIOType, SecretUtils


'''SETTINGS'''
SPARK_FREE_API_KEYS = {
    'lite': [
        'BrQGpZuyF-r-aa8BsEcILHhyck6bRGmeW5tOjGlKwLNkKeU04juloTRFZP7GtRlDWcU4hPcHtTycdJfRfcktr1JWlj54hlfnTe7bbbKl-FprpUcj',
        'D9-CjAmX4ww-_QFr_s9BLQ1fHoq60aHZ46-ATnsHL4JeO5_0GeQeEKP9Di337RQ1n2VQz0U-MhNZcaMn7XN-v_MJDkhQ9zIONBCkstyOyvgRePzp', 
    ],
    '4.0Ultra': [
        {'app_id': 'z8LR_zVlO-rnQzbiO_LoXe8wx4iPjRkFAegZ57flf1Xs6LIKwI4o2A==', 'api_key': '7fgq2apXPXAMJZ7t9ECF6st-jKQPxflqgaj7Z276GWgZkV_zivo-0SY4a0aIzD9-ZT-mBWLMBd9Eq2s8-NpBpI5Dx1UcSFfg', 'api_secret': 'Ly3BhBZeA19xT_D2HBlOhklLUw7JsljAJM1Cr4M9Z-JFQv_9O12ZQsPlp4FMy_9MNNxM8zmHnAMDqm8oQsiimUyL-h-CsNMw'}
    ],
}


'''IFLYTEKSparkEndpoints'''
class IFLYTEKSparkEndpoints(BaseEndpoint):
    provider, model = "iflytek", "spark"
    def __init__(self, **kwargs):
        super(IFLYTEKSparkEndpoints, self).__init__(**kwargs)
        self.registervariant("lite", io_supported=[ModelIOType.fromtag("T2T")])
        self.registerapi(
            version="lite", name="cggapi", io=ModelIOType.fromtag("T2T"), handler="cggapi",
            priority=10, note='thirdparty api: https://api.cenguigui.cn/api/chat/?msg=',
        )
        self.registerapi(
            version="lite", name="officialapiv1", io=ModelIOType.fromtag("T2T"), handler="officialapiv1",
            priority=100, note='official api: https://spark-api-open.xf-yun.com/v1'
        )
        self.registervariant("4.0Ultra", io_supported=[ModelIOType.fromtag("T2T")])
        self.registerapi(
            version="4.0Ultra", name="officialapiv4", io=ModelIOType.fromtag("T2T"), handler="officialapiv4",
            priority=100, note='official api: wss://spark-api.xf-yun.com/v4.0/chat'
        )
    '''officialapiv1'''
    def officialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://spark-api-open.xf-yun.com/v1', candidate_api_keys=SPARK_FREE_API_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''officialapiv4'''
    def officialapiv4(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        # init
        request_overrides = request_overrides or {}
        full_response, websocket_cfg = [], req.websocket
        # _buildauthurl
        def _buildauthurl(ws_url: str, api_key: str, api_secret: str) -> str:
            u = urlparse(ws_url)
            host, path = u.netloc, u.path
            date = format_date_time(mktime(datetime.now().timetuple()))
            signature_origin = f"host: {host}\n" f"date: {date}\n" f"GET {path} HTTP/1.1"
            signature_sha = hmac.new(api_secret.encode("utf-8"), signature_origin.encode("utf-8"), digestmod=hashlib.sha256).digest()
            signature = base64.b64encode(signature_sha).decode("utf-8")
            authorization_origin = f'api_key="{api_key}", algorithm="hmac-sha256", headers="host date request-line", signature="{signature}"'
            authorization = base64.b64encode(authorization_origin.encode("utf-8")).decode("utf-8")
            params = {"authorization": authorization, "date": date, "host": host}
            return ws_url + "?" + urlencode(params)
        # _buildrequest
        def _buildrequest(websocket_cfg: dict, content: str, domain: str, app_id: str) -> dict:
            return {
                "header": {"app_id": app_id, "uid": "user_001"},
                "parameter": websocket_cfg.get('parameter') or {"chat": {"domain": domain, "temperature": 0.5, "top_k": 4, "max_tokens": 2048}},
                "payload": websocket_cfg.get('payload') or {"message": {"text": [{"role": "user", "content": content}]}},
            }
        # _onopen
        def _onopen(ws: websocket.WebSocketApp):
            payload = _buildrequest(content=req.text, domain=version, app_id=app_id, websocket_cfg=websocket_cfg)
            ws.send(json.dumps(payload, ensure_ascii=False))
        # _onmessage
        def _onmessage(ws: websocket.WebSocketApp, message):
            nonlocal full_response
            data: dict = json.loads(message)
            header: dict = data.get("header", {})
            if header.get("code", -1) != 0: raise RuntimeError(f"Spark error: {header}")
            payload: dict = data.get("payload", {})
            choices: dict = payload.get("choices", {})
            text_list: list[dict] = choices.get("text", [])
            if text_list:
                delta = text_list[0].get("content", "")
                if delta: full_response.append(delta)
            if header.get("status") == 2: ws.close()
        # _onerror
        def _onerror(ws, error): print("\n[ws error]", error)
        # _onclose
        def _onclose(ws, close_status_code, close_msg): pass
        # obtain api info
        app_secret = random.choice(SPARK_FREE_API_KEYS[version])
        app_id = SecretUtils.b64decode(SecretUtils.decryptaesgcm(app_secret['app_id'], self.aes_gem_key))
        api_key = SecretUtils.b64decode(SecretUtils.decryptaesgcm(app_secret['api_key'], self.aes_gem_key))
        api_secret = SecretUtils.b64decode(SecretUtils.decryptaesgcm(app_secret['api_secret'], self.aes_gem_key))
        # websocket
        ws_final_url = _buildauthurl("wss://spark-api.xf-yun.com/v4.0/chat", api_key, api_secret)
        ws = websocket.WebSocketApp(ws_final_url, on_open=_onopen, on_message=_onmessage, on_error=_onerror, on_close=_onclose)
        ws.run_forever(sslopt={"check_hostname": True})
        # return
        return ChatResponse(text="".join(full_response), raw=full_response)
    '''cggapi'''
    def cggapi(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        request_overrides = request_overrides or {}
        try:
            resp = self.session.get("https://api.cenguigui.cn/api/chat/", params={"msg": req.text}, **request_overrides)
            resp.raise_for_status()
        except:
            resp = self.session.get("https://api-v1.cenguigui.cn/api/chat/", params={"msg": req.text}, **request_overrides)
            resp.raise_for_status()
        api_resp_raw_data = resp2json(resp)
        return ChatResponse(text=api_resp_raw_data['data']['content'], raw=api_resp_raw_data)


'''
Function:
    Implementation of BaseEndpoint and EndpointRegistry
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import httpx
import base64
import random
import requests
from freeproxy import freeproxy
from openai import OpenAI, DefaultHttpxClient
from typing import Optional, Dict, Iterable, List, Any, Union, Tuple
from ..utils import ModelVariant, ChatRequest, ChatResponse, Modality, ModelIOType, APISpec, SecretUtils


'''BaseEndpoint'''
class BaseEndpoint:
    provider: str = "unknown"
    model: str = "unknown"
    def __init__(self, default_headers: dict = None, auto_set_proxies: bool = False,  proxy_sources: list = None, default_version: str = "default", aes_gem_key: bytes = None):
        assert aes_gem_key is not None, 'aes_gem_key is not set, so this library cannot be used. Please follow the WeChat official account "Charles的皮卡丘" and reply "FreeGPTHub" to obtain it.'
        # init session
        self.session = requests.Session()
        self.session.headers.update(default_headers or {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/143.0.0.0 Safari/537.36"
        })
        # set attributes
        self.aes_gem_key = aes_gem_key
        self.variants: Dict[str, ModelVariant] = {}
        self.auto_set_proxies = auto_set_proxies
        self.proxy_sources = proxy_sources
        self.default_version = default_version
        # proxied_session_client
        self.proxied_session_client = freeproxy.ProxiedSessionClient(
            proxy_sources=['GeonodeProxiedSession'] if proxy_sources is None else proxy_sources, 
            disable_print=True
        ) if auto_set_proxies else None
    '''registervariant'''
    def registervariant(self, version: str, io_supported: Iterable[ModelIOType]):
        if version in self.variants: return
        self.variants[version] = ModelVariant(provider=self.provider, model=self.model, version=version, io_supported=frozenset(io_supported), apis={})
    '''registerapi'''
    def registerapi(self, *, version: str, name: str, io: ModelIOType, handler: str, priority: int = 0, note: str = ""):
        if version not in self.variants: raise ValueError(f"Variant '{version}' not registered. Call registervariant() first.")
        if not hasattr(self, handler): raise AttributeError(f"Handler method '{handler}' not found on {self.__class__.__name__}")
        v = self.variants[version]
        v.apis[name] = APISpec(name=name, io=io, handler=handler, priority=priority, note=note)
    '''inferio'''
    def inferio(self, req: Any) -> ModelIOType:
        if isinstance(req, ChatRequest):
            in_mods: List[Union[Modality, str]] = [Modality.TEXT]
            if req.images: in_mods.append(Modality.IMAGE)
            return ModelIOType.create(inputs=in_mods, outputs=[Modality.TEXT])
        raise TypeError(f"Unsupported request type: {type(req)}")
    '''initopenaiclient'''
    def initopenaiclient(self, base_url: str, candidate_api_keys: dict, req: ChatRequest, version: str = None):
        openai_cfg: dict = req.openai
        if 'http_client' not in openai_cfg['client'] and self.auto_set_proxies:
            client = OpenAI(
                base_url=base_url, api_key=SecretUtils.b64decode(SecretUtils.decryptaesgcm(random.choice(candidate_api_keys[version]), self.aes_gem_key)), 
                http_client=DefaultHttpxClient(
                    proxy=self.proxied_session_client.getrandomproxy(proxy_format='freeproxy'), 
                    transport=httpx.HTTPTransport(local_address="0.0.0.0")
                ), **openai_cfg['client']
            )
        else:
            client = OpenAI(
                base_url=base_url, api_key=SecretUtils.b64decode(SecretUtils.decryptaesgcm(random.choice(candidate_api_keys[version]), self.aes_gem_key)), **openai_cfg['client']
            )
        return client
    '''openaisdk'''
    def openaisdk(self, base_url: str, candidate_api_keys: dict, api_family: str, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        # init
        request_overrides = request_overrides or {}
        openai_cfg: dict = req.openai
        # create client
        client = self.initopenaiclient(base_url=base_url, candidate_api_keys=candidate_api_keys, req=req, version=version)
        # specify model
        model_override = openai_cfg[api_family].pop('model', None)
        model = model_override or version
        # construct message
        messages_override = openai_cfg[api_family].pop('messages', None)
        messages = messages_override or [{"role": "user", "content": req.text}]
        # request based on api_family
        if api_family in ['client.chat.completions.create']:
            resp = client.chat.completions.create(model=model, messages=messages, **openai_cfg[api_family])
            text = self._extracttextfromchatcompletionsobj(resp)
            return ChatResponse(text=text, raw=resp.to_json() or resp)
        elif api_family in ['client.responses.create']:
            resp = client.responses.create(model=model, input=req.text, **openai_cfg[api_family])
            text = self._extracttextfromresponseobj(resp)
            raw = resp.model_dump() if hasattr(resp, "model_dump") else resp
            return ChatResponse(text=text, raw=raw)
        else:
            raise TypeError(f"Unsupported api_family: {api_family}")
    '''visionopenaisdk'''
    def visionopenaisdk(self, base_url: str, candidate_api_keys: dict, api_family: str, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        # init
        request_overrides = request_overrides or {}
        openai_cfg: dict = req.openai
        # create client
        client = self.initopenaiclient(base_url=base_url, candidate_api_keys=candidate_api_keys, req=req, version=version)
        # specify model
        model_override = openai_cfg[api_family].pop('model', None)
        model = model_override or version
        # construct message
        messages_override = openai_cfg[api_family].pop('messages', None)
        # request based on api_family
        if api_family in ['client.chat.completions.create']:
            images = getattr(req, "images", None) or ()
            content = [{"type": "text", "text": req.text}]
            for img in images: content.append({"type": "image_url", "image_url": {'url': self._imgtourl(img)}})
            messages = messages_override or [{"role": "user", "content": content}]
            resp = client.chat.completions.create(model=model, messages=messages, **openai_cfg[api_family])
            text = self._extracttextfromchatcompletionsobj(resp)
            return ChatResponse(text=text, raw=resp.to_json() or resp)
        elif api_family in ['client.responses.create']:
            images = getattr(req, "images", None) or ()
            content = [{"type": "input_text", "text": req.text}]
            for img in images: content.append({"type": "input_image", "image_url": self._imgtourl(img)})
            messages = messages_override or [{"role": "user", "content": content}]
            resp = client.responses.create(model=model, input=messages, **openai_cfg[api_family])
            text = self._extracttextfromresponseobj(resp)
            raw = resp.model_dump() if hasattr(resp, "model_dump") else resp
            return ChatResponse(text=text, raw=raw)
        else:
            raise TypeError(f"Unsupported api_family: {api_family}")
    '''send'''
    def send(self, req: Any, api: Optional[str] = None, version: Optional[str] = None, request_overrides: dict = None) -> ChatResponse:
        # set proxies
        if self.auto_set_proxies:
            try: self.session.proxies = self.proxied_session_client.getrandomproxy()
            except: self.session.proxies = {}
        # model version
        version = version or self.default_version
        if version not in self.variants: raise ValueError(f"Unknown version '{version}'. Available: {list(self.variants.keys())}")
        # variant
        variant, req_io = self.variants[version], self.inferio(req)
        # assert io
        if req_io not in variant.io_supported: raise ValueError(f"Variant {variant.id} does not support {req_io.detailedtag()}")
        # with specified api
        if api is not None:
            spec = variant.apis.get(api)
            if spec is None: raise ValueError(f"Unknown api '{api}'. Available: {list(variant.apis.keys())}")
            if spec.io != req_io: raise ValueError(f"API '{api}' does not support {req_io.detailedtag()}, supports {spec.io.detailedtag()}")
            out: ChatResponse = getattr(self, spec.handler)(req, request_overrides=request_overrides, version=version)
            return ChatResponse(text=out.text, raw=out.raw, api_used=spec.name, variant_used=variant.id)
        # without specified api, select the best one
        candidates = [s for s in variant.apis.values() if s.io == req_io]
        if not candidates: raise ValueError(f"No API supports {req_io.detailedtag()} in variant {variant.id}")
        best = max(candidates, key=lambda s: s.priority)
        out: ChatResponse = getattr(self, best.handler)(req, request_overrides=request_overrides, version=version)
        # return
        return ChatResponse(text=out.text, raw=out.raw, api_used=best.name, variant_used=variant.id)
    '''describe'''
    def describe(self) -> Dict[str, Any]:
        return {
            "provider": self.provider, "model": self.model, "default_version": self.default_version,
            "versions": {
                ver: {
                    "variant_id": v.id, "io_supported": [io.detailedtag() for io in v.io_supported],
                    "apis": {name: {"io": spec.io.detailedtag(), "priority": spec.priority, "note": spec.note, "handler": spec.handler} for name, spec in v.apis.items()},
                } for ver, v in self.variants.items()
            },
        }
    '''_extracttextfromresponseobj'''
    @staticmethod
    def _extracttextfromresponseobj(resp: Any) -> str:
        txt = getattr(resp, "output_text", None)
        if isinstance(txt, str) and txt.strip(): return txt.strip()
        out = getattr(resp, "output", None) or []
        texts: List[str] = []
        for item in out:
            if getattr(item, "type", None) != "message" or getattr(item, "role", None) != "assistant": continue
            for part in getattr(item, "content", None) or []:
                if getattr(part, "type", None) == "output_text":
                    t = getattr(part, "text", "")
                    if isinstance(t, str) and t.strip(): texts.append(t.strip())
        return "\n".join(texts).strip()
    '''_extracttextfromchatcompletionsobj'''
    @staticmethod
    def _extracttextfromchatcompletionsobj(resp: Any) -> str:
        choices, texts = resp.choices, []
        for item in choices:
            if not hasattr(item, 'message') or item.message.role not in ["assistant"]: continue
            texts.append(item.message.content)
        return "\n".join(texts).strip()
    '''_imgtourl'''
    @staticmethod
    def _imgtourl(img: Union[str, bytes, Tuple[bytes, str]]) -> str:
        if isinstance(img, str):
            s = img.strip()
            if s.startswith("http://") or s.startswith("https://") or s.startswith("data:"): return s
            raise ValueError("Image str must be http(s) URL or data: URL")
        if isinstance(img, tuple):
            b, mime = img
            b64 = base64.b64encode(b).decode("utf-8")
            return f"data:{mime};base64,{b64}"
        if isinstance(img, (bytes, bytearray)):
            b64 = base64.b64encode(bytes(img)).decode("utf-8")
            return f"data:image/png;base64,{b64}"
        raise TypeError(f"Unsupported image type: {type(img)}")


'''EndpointRegistry'''
class EndpointRegistry:
    def __init__(self):
        self._endpoints: List[BaseEndpoint] = []
    '''register'''
    def register(self, endpoint: BaseEndpoint):
        self._endpoints.append(endpoint)
    '''listall'''
    def listall(self) -> List[Dict[str, Any]]:
        return [ep.describe() for ep in self._endpoints]
    '''findvariantsbyio'''
    def findvariantsbyio(self, io: Union[str, ModelIOType]) -> List[Tuple[BaseEndpoint, str]]:
        target = ModelIOType.fromtag(io) if isinstance(io, str) else io
        hits: List[Tuple[BaseEndpoint, str]] = []
        for ep in self._endpoints:
            for ver, v in ep.variants.items():
                if target in v.io_supported: hits.append((ep, ver))
        return hits
    '''get'''
    def get(self, provider: str, model: str) -> List[BaseEndpoint]:
        return [ep for ep in self._endpoints if ep.provider == provider and ep.model == model]


'''
Function:
    Implementation of ModelScopeEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from ..providers import BaseEndpoint
from ..utils import ChatRequest, ChatResponse, ModelIOType, Modality


'''SETTINGS'''
MODELSCOPE_FREE_SHARED_KEYS = [
    'd8piqH1om7slDpWVFr9fABSIFM3XCG2Ra4SMy_tfjnklvf9UqO6_ihRsGxTeobKIJEY8F-nmAsYFkKOPuOOfdB5SmwLGaVRobr-WYcsuNz4=', 
    'oydsr5t5wmt-rOSTfatD3XtYBzfRFFpnHzrr4NlUBPkSkEqEqGhOEuyPQ4ktEV6w2lfKjDzFKkLBgaAQXEkfAoQpNdA0lbwSw6VbjLyCLNA=', 
    'YtNDaEMdeCCcKi-Yy1sSi6o-lfz5hzSLog8mANsIdIKzLZzaD4X8Da_5mCAzMfNVkJvh6rwsBfRiSh6R9TWdH4T26rM6x5JRLlXb6oITkLk=', 
    '__ZVhAz8E8DaFnvPEA5zOSaq_GckwEWMhrjs2CIUPx6REQpiZNFsgIK6isg7GhewToXfXgd6Iz9oXWUrZG28SdYJUDfMqRAKqHvKtIMmScQ=', 
    '_DdGpBoqyuFvDRIta6S1UhtjrmED9fI2ZY3kHR2ama4PWsMp49z4sAce-fqHevqF0XXAYHLVxcz_hjYGyzWHg0YElNdlw0VOveFmiRsWdqI=', 
    'pHZF58cVvgZAniBVIrcvwQQ-S7TmBzqf4N901Xpu0IGX-HFmviIiAUuMqXUKi4JhvF5dG8W3C8TRNQcjQGoFq2UwfQl6iW_KdqvWYRNTf7w=', 
    'XS67Yk6npZKPd9BeE-Z7ukUBylE9wlCFhNguP78MlxoTYv3M1hcctcLfNEcrfsSHnT88VT0MWQfCbjvF7exTBP4c80wCuyq-ca9coc2VqtE=', 
    'SGa6z9tEpQpILJ4irk7-mvC8DUHSnRf8CS1upuNdzyDQoITLDnnaml_l8F7V_fYzBYrGHvbLJ-TDKojqO3iPyoJpEJZGzoWZg4gikvLpD4Y=', 
    'L0Qsd768J0yA5Ko_DIdVcluFSJcDmSROBXjTaj9p33R0IZ5EIQ3ihKshMUd2UyzubJFTySSw2nLMA2WVxnzbsP1ckMAu--z7MuHtt-f88s8=', 
    'f3EKoSKBfLAfxmF2faH7T9zWziG0V-Bm4XaDZgEdsBgfKoQFo-7HuggxK4dXmmRiF1AmIeNel65mU4bsJ-FY7hQPEd9NMCHdqTv4_WGRwGM=', 
    'xrufyy485ATw0B_gjX7WHowLGlbmUSapivx2t0nXuSy7DEQTofKV6-Dahc1_TVNFgEtcDd6xpFj3CPNq3rWEipqV-VVrhbCvxU4BFsQ2NYs=', 
    '0cKSIfmeU08Ta9QY9sQWb8Rl5xW4vR2ktVpPcdXa82HonK0L6Fz23kJO7sb_4zTjhCS1Sw_UjxqNImAUT3uioN_zUcEkViv7s3DIptI3w3Q=', 
    '7RwLU4FOqfvJcgOlX9csIhcYpK94DeJTnb13Xk0Nol_9OjgABW4Xc6VElB3ucl5XEkW8oqqBMIVtGp4eMB5uQt-0eMCD5pKPkhFwUx8v5Vg=', 
    '_DG9pFj8UGd4R1UEXj1Z623TPiEWIl2aolwkuuRD1P_JA6WqUnIbiYUQM-Ic1mqOkYkgMDIhvpjipeajNcmFZhMMTG1A3RqVKUhbQ1i3Qvg=', 
    'C-4l68NbnLfOnJ3ygysIKjQy-j43qhZymt0SWCxnnel4QpLrH3oU45v2n1bU_i2UrC42PMbA-HxYKzlgt2BcWvNJaFDYxOMc4fGsVPPr2CM=', 
    'zK-WLS_jjbdJM789aUPaqBwQiUroNqn8OMSFGehv1gyentkX-94on_XXlOqf6NBbCbQ9Djc6SueKwQuiAfW9iqzkEMgIWIsOJ_2AL1sdtiQ=', 
    'X6Q0dRVwgRuOpZLqy9hx5UcWK9_Ie4D4VPRXQn8F-kBtR_6cW5XabAmsyiyJ5Pu2OnlxFhi56SeR8xpVPHL5K25LY_qnJv3QsDMiyVIh2CE=',
]
MODELSCOPE_FREE_MODEL_IDS = [
    'deepseek-ai/DeepSeek-R1-0528', 'deepseek-ai/DeepSeek-R1-Distill-Llama-70B', 'deepseek-ai/DeepSeek-R1-Distill-Llama-8B', 'deepseek-ai/DeepSeek-R1-Distill-Qwen-1.5B', 'deepseek-ai/DeepSeek-R1-Distill-Qwen-14B', 'deepseek-ai/DeepSeek-R1-Distill-Qwen-32B', 'deepseek-ai/DeepSeek-R1-Distill-Qwen-7B', 'deepseek-ai/DeepSeek-V3.2', 
    'LLM-Research/c4ai-command-r-plus-08-2024', 'LLM-Research/Llama-4-Maverick-17B-128E-Instruct', 
    'Menlo/Jan-nano', 'MiniMax/MiniMax-M1-80k', 
    'mistralai/Ministral-8B-Instruct-2410', 'mistralai/Mistral-Large-Instruct-2407', 'mistralai/Mistral-Small-Instruct-2409', 
    'MusePublic/Qwen-Image-Edit', 
    'opencompass/CompassJudger-1-32B-Instruct', 
    'OpenGVLab/InternVL3_5-241B-A28B', 
    'PaddlePaddle/ERNIE-4.5-0.3B-PT', 'PaddlePaddle/ERNIE-4.5-21B-A3B-PT', 'PaddlePaddle/ERNIE-4.5-300B-A47B-PT', 'PaddlePaddle/ERNIE-4.5-VL-28B-A3B-PT', 
    'Qwen/QVQ-72B-Preview', 'Qwen/Qwen-Image-Edit', 'Qwen/Qwen2.5-14B-Instruct', 'Qwen/Qwen2.5-14B-Instruct-1M', 'Qwen/Qwen2.5-32B-Instruct', 'Qwen/Qwen2.5-72B-Instruct', 'Qwen/Qwen2.5-7B-Instruct', 'Qwen/Qwen2.5-7B-Instruct-1M', 'Qwen/Qwen2.5-Coder-14B-Instruct', 'Qwen/Qwen2.5-Coder-32B-Instruct', 'Qwen/Qwen2.5-Coder-7B-Instruct', 'Qwen/Qwen2.5-VL-32B-Instruct', 'Qwen/Qwen2.5-VL-3B-Instruct', 'Qwen/Qwen2.5-VL-72B-Instruct', 'Qwen/Qwen2.5-VL-7B-Instruct', 'Qwen/Qwen3-0.6B', 'Qwen/Qwen3-1.7B', 'Qwen/Qwen3-14B', 'Qwen/Qwen3-235B-A22B', 'Qwen/Qwen3-235B-A22B-Instruct-2507', 'Qwen/Qwen3-235B-A22B-Thinking-2507', 'Qwen/Qwen3-30B-A3B', 'Qwen/Qwen3-30B-A3B-Thinking-2507', 'Qwen/Qwen3-32B', 'Qwen/Qwen3-4B', 'Qwen/Qwen3-8B', 'Qwen/Qwen3-Coder-30B-A3B-Instruct', 'Qwen/Qwen3-Coder-480B-A35B-Instruct', 'Qwen/Qwen3-Next-80B-A3B-Instruct', 'Qwen/Qwen3-Next-80B-A3B-Thinking', 'Qwen/Qwen3-VL-235B-A22B-Instruct', 'Qwen/Qwen3-VL-8B-Instruct', 'Qwen/Qwen3-VL-8B-Thinking', 'Qwen/QwQ-32B', 'Qwen/QwQ-32B-Preview', 
    'Shanghai_AI_Laboratory/Intern-S1', 'Shanghai_AI_Laboratory/Intern-S1-mini', 
    'stepfun-ai/step3', 
    'XGenerationLab/XiYanSQL-QwenCoder-32B-2412', 'XGenerationLab/XiYanSQL-QwenCoder-32B-2504', 
    'XiaomiMiMo/MiMo-V2-Flash'
]
MODELSCOPE_FREE_TI_MODEL_IDS = [
    "LLM-Research/Llama-4-Maverick-17B-128E-Instruct",
    "OpenGVLab/InternVL3_5-241B-A28B",
    "PaddlePaddle/ERNIE-4.5-VL-28B-A3B-PT",
    "Qwen/QVQ-72B-Preview", "Qwen/Qwen2.5-VL-32B-Instruct", "Qwen/Qwen2.5-VL-3B-Instruct", "Qwen/Qwen2.5-VL-72B-Instruct", "Qwen/Qwen2.5-VL-7B-Instruct", "Qwen/Qwen3-VL-235B-A22B-Instruct", "Qwen/Qwen3-VL-8B-Instruct", "Qwen/Qwen3-VL-8B-Thinking",
    "stepfun-ai/step3",
]
MODELSCOPE_FREE_API_V1_KEYS = {}
for model_id in MODELSCOPE_FREE_MODEL_IDS: MODELSCOPE_FREE_API_V1_KEYS[model_id] = MODELSCOPE_FREE_SHARED_KEYS


'''ModelScopeEndpoints'''
class ModelScopeEndpoints(BaseEndpoint):
    provider, model = "modelscope", "common"
    def __init__(self, **kwargs):
        super(ModelScopeEndpoints, self).__init__(**kwargs)
        TI2T = ModelIOType.create(inputs=[Modality.TEXT, Modality.IMAGE], outputs=[Modality.TEXT])
        for ver in list(MODELSCOPE_FREE_API_V1_KEYS.keys()):
            if ver in MODELSCOPE_FREE_TI_MODEL_IDS:
                self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T"), TI2T])
                self.registerapi(
                    version=ver, name="officialapiv1", io=ModelIOType.fromtag("T2T"), handler="officialapiv1",
                    priority=100, note='official api: https://api-inference.modelscope.cn/v1'
                )
                self.registerapi(
                    version=ver, name="visionofficialapiv1", io=TI2T, handler="visionofficialapiv1", priority=100, note='official api: https://api-inference.modelscope.cn/v1'
                )
            else:
                self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
                self.registerapi(
                    version=ver, name="officialapiv1", io=ModelIOType.fromtag("T2T"), handler="officialapiv1",
                    priority=100, note='official api: https://api-inference.modelscope.cn/v1'
                )
    '''officialapiv1'''
    def officialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://api-inference.modelscope.cn/v1', candidate_api_keys=MODELSCOPE_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )
    '''visionofficialapiv1'''
    def visionofficialapiv1(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.visionopenaisdk(
            base_url='https://api-inference.modelscope.cn/v1', candidate_api_keys=MODELSCOPE_FREE_API_V1_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )


'''
Function:
    Implementation of Common Utils
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
import requests
import json_repair


'''resp2json'''
def resp2json(resp: requests.Response):
    if not isinstance(resp, requests.Response): return {}
    try:
        result = resp.json()
    except:
        result = json_repair.loads(resp.text)
    if not result: result = dict()
    return result


'''title'''
__title__ = 'freegpthub'
'''description'''
__description__ = 'FreeGPTHub: A truly free, unified GPT API gateway—unlike “fake-free” alternatives that hide paywalls behind quotas, trials, or mandatory top-ups.'
'''url'''
__url__ = 'https://github.com/CharlesPikachu/FreeGPTHub'
'''version'''
__version__ = '0.1.0'
'''author'''
__author__ = 'Zhenchao Jin'
'''email'''
__email__ = 'charlesblwx@gmail.com'
'''license'''
__license__ = 'Apache License 2.0'
'''copyright'''
__copyright__ = 'Copyright 2025-2030 Zhenchao Jin'


'''init'''
from .registered import REGISTERED_ENDPOINTS
from .gpthub import (
    ModelIOType, Modality, ChatRequest, ChatResponse, APISpec, ModelVariant, SecretUtils, StreamSanitizer,
    BaseEndpoint, EndpointRegistry, IFLYTEKSparkEndpoints, BaiduQianfanEndpoints, OpenAIChatGPTEndpoints, ZhipuGLMEndpoints,
    HighflyerDeepSeekEndpoints, ByteDanceDoubaoEndpoints, AlibabaQwenEndpoints, MiniMaxEndpoints, ModelScopeEndpoints,
)


'''
Function:
    Implementation of BaiduQianfanEndpoints
Author:
    Zhenchao Jin
WeChat Official Account (微信公众号):
    Charles的皮卡丘
'''
from .base import BaseEndpoint
from ..utils import ChatRequest, ChatResponse, ModelIOType


'''SETTINGS'''
QIANFAN_FREE_SHARED_KEYS = [
    'oiP1BpFuiPBfnyHJm9i5TAgwMZG3Y0Vd8UBqTmxaS2M7MNVHtz14eNsaOKa5n2T-2DDthiKYFZU25JAzlgUAECa4KIrn5s08Xete_El9TPTOTDPTvzsQaDpE-5JSrTYYrmPaWgmp7hYcfEIhYk9nsl62zhMcBLIk_7gXV9AXfb8=', 
    'DTeSqHsTdBZthSJfrAR575nGNh4ZGNk0Jz-hU5YrMZicCeJuxUJDO9YJACGcTD9UQ60f41eYwv3Gl6okp7hT3LYHTClIApx_nsOSVty2_D-BViS9qTqkQuh-ZZZSZOfWrBtglkl7I03ZQW6o1bug3uq75HEjtBMeK-dWtQJTlvE=', 
    'mHXHSnXsn562akeQfJDEvheYzdMtzVXuhLh-I8_NRjn5_yB0R8zGZpWaXM7rlN8SbL-kneABpK5FsMWWMm2X0ClHX3qqBjFGALf6EXzcpuIj6H2wcyklxHmn0i2AZnaGpxffSKuyoARrgY0lmokpMI0o_1OoxHbIeykhoyCW7R8='
]
QIANFAN_FREE_API_V2_KEYS = {'ernie-4.0-turbo-8k': QIANFAN_FREE_SHARED_KEYS}


'''BaiduQianfanEndpoints'''
class BaiduQianfanEndpoints(BaseEndpoint):
    provider, model = "baidu", "qianfan"
    def __init__(self, **kwargs):
        super(BaiduQianfanEndpoints, self).__init__(**kwargs)
        for ver in list(QIANFAN_FREE_API_V2_KEYS.keys()):
            self.registervariant(ver, io_supported=[ModelIOType.fromtag("T2T")])
            self.registerapi(
                version=ver, name="officialapiv2", io=ModelIOType.fromtag("T2T"), handler="officialapiv2",
                priority=100, note='official api: https://qianfan.baidubce.com/v2'
            )
    '''officialapiv2'''
    def officialapiv2(self, req: ChatRequest, request_overrides: dict = None, version: str = None) -> ChatResponse:
        return self.openaisdk(
            base_url='https://qianfan.baidubce.com/v2', candidate_api_keys=QIANFAN_FREE_API_V2_KEYS, api_family='client.chat.completions.create',
            req=req, request_overrides=request_overrides, version=version
        )